from enum import Enum
from wizzi_utils.models.labels_bank import *


class ModelType(Enum):
    OdCvNormal = 'Cv2OdModel'
    OdTflNormal = 'TflOdModel'
    PdCvNormal = 'Cv2PdModel'
    PdTflNormal = 'TflPdModel'
    PdTflPoseNet = 'TflPdModelPoseNet'
    PdCvCocoMulti = 'Cv2PdModelCocoMultiPoses'


class Jobs(Enum):
    OBJECT_DETECTION = 'object_detection'
    SEGMENTATION = 'segmentation'  # future
    POSE_DETECTION = 'pose_detection'
    CLASSIFICATION = 'classification'  # future


class DnnFamily(Enum):
    Caffe = 'Caffe'
    Darknet = 'Darknet'
    TF = 'TensorFlow'


class DownloadStyle(Enum):
    Direct = 'Direct'
    Tar = 'tar'
    Zip = 'zip'


MODELS_CONFIG = {
    # object detection TFL and cv2
    # TFL
    'coco_ssd_mobilenet_v1_1_0_quant_2018_06_29': {
        'model_type': ModelType.OdTflNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'threshold': 0.2,
        'nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'tflite': {
            'url': 'http://storage.googleapis.com/download.tensorflow.org/models/tflite/' +
                   'coco_ssd_mobilenet_v1_1.0_quant_2018_06_29.zip',
            'download_style': DownloadStyle.Zip.value,
            'file_to_look': 'detect.tflite',
        },
        'extra': {
            'info': 'https://gist.github.com/iwatake2222/e4c48567b1013cf31de1cea36c4c061c',
            'size': 'tflite(3.99 MB)',
            'fps': '5.10',
        },
    },
    'ssd_mobilenet_v3_small_coco_2020_01_14': {
        'model_type': ModelType.OdTflNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'threshold': 0.2,
        'nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,  # TODO labels are not correct
        'tflite': {
            'url': 'http://download.tensorflow.org/models/object_detection/' +
                   'ssd_mobilenet_v3_small_coco_2020_01_14.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'model.tflite',
        },
        'extra': {
            'web_name': 'ssd_mobilenet_v3_small_coco_2020_01_14',
            'size': 'tflite(6.86 MB)',
            'fps': '21.96 FPS',
            'info': 'https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/'
                    + 'tf1_detection_zoo.md#mobile-models'
        },
    },
    'ssd_mobilenet_v3_large_coco_2020_01_14': {
        'model_type': ModelType.OdTflNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'threshold': 0.2,
        'nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'tflite': {
            'url': 'http://download.tensorflow.org/models/object_detection/' +
                   'ssd_mobilenet_v3_large_coco_2020_01_14.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'model.tflite',
        },
        'extra': {
            'web_name': 'ssd_mobilenet_v3_large_coco_2020_01_14',
            'size': 'tflite(12.42 MB)',
            'fps': '12.94 FPS',
            'info': 'https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/'
                    + 'tf1_detection_zoo.md#mobile-models',
        },
    },
    'ssd_mobilenet_v2_mnasfpn': {
        'model_type': ModelType.OdTflNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'threshold': 0.2,
        'nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'tflite': {
            'url': 'http://download.tensorflow.org/models/object_detection/' +
                   'ssd_mobilenet_v2_mnasfpn_shared_box_predictor_320x320_coco_sync_2020_05_18.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'model.tflite',
        },
        'extra': {
            'web_name': 'ssd_mobilenet_v2_mnasfpn',
            'size': 'tflite(9.68 MB)',
            'fps': '5.65 FPS',
            'info': 'https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/'
                    + 'tf1_detection_zoo.md#mobile-models'
        },
    },
    'ssdlite_mobiledet_cpu_320x320_coco_2020_05_19': {
        'model_type': ModelType.OdTflNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'threshold': 0.2,
        'nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'tflite': {
            'url': 'http://download.tensorflow.org/models/object_detection/' +
                   'ssdlite_mobiledet_cpu_320x320_coco_2020_05_19.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'model.tflite',
        },
        'extra': {
            'web_name': 'ssdlite_mobiledet_cpu_320x320_coco_2020_05_19',
            'size': 'tflite(15.97 MB)',
            'fps': '11.01 FPS',
            'info': 'https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/'
                    + 'tf1_detection_zoo.md#mobile-models'
        },
    },
    'ssd_mobilenet_v1_1_metadata_1': {
        'model_type': ModelType.OdTflNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'threshold': 0.2,
        'nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'tflite': {
            'url': 'https://tfhub.dev/tensorflow/lite-model/ssd_mobilenet_v1/1/metadata/1?lite-format=tflite',
            'download_style': DownloadStyle.Direct.value,
        },
        'URL': {
            'tflite': 'https://tfhub.dev/tensorflow/lite-model/ssd_mobilenet_v1/1/metadata/1?lite-format=tflite',
            'tflite_download_style': DownloadStyle.Direct.value,

        },
        'extra': {
            'web_name': 'TODO',
            'size': 'tflite(3.99 MB)',
            'fps': '4.43 FPS',
            'info': 'https://www.tensorflow.org/lite/examples/object_detection/overview'
        },
    },
    'ssdlite_mobilenet_v2_coco_300_integer_quant_with_postprocess': {
        'model_type': ModelType.OdTflNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'threshold': 0.2,
        'nms': {
            'score_threshold': 0.4,
            'nms_threshold': 0.4,
        },
        'labels_dict': COCO_80_RAW,
        'tflite': {
            'url': 'https://drive.google.com/' +
                   'uc?export=download&confirm=${CODE}&id=1LjTqn5nChAVKhXgwBUp00XIKXoZrs9sB',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'ssdlite_mobilenet_v2_coco_300_integer_quant_with_postprocess',
            'size': 'tflite(5.09 MB)',
            'fps': '0.63 FPS',
            'info': 'https://github.com/PINTO0309/PINTO_model_zoo/tree/main/006_mobilenetv2-ssdlite/01_coco/'
                    + '03_integer_quantization'
        },
    },
    # CV
    'yolov4_tiny': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (608, 608),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'cfg': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/yolov4-tiny.cfg',
            'download_style': DownloadStyle.Direct.value,
        },
        'weights': {
            'url': 'https://github.com/AlexeyAB/darknet/releases/download/darknet_yolo_v4_pre/yolov4-tiny.weights',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'yolov4-tiny',
            'size': 'weights(23.13 MB), cfg(2.96 KB)',
            'fps': '11.33',
        },
    },
    'yolov4': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (608, 608),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'cfg': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/yolov4.cfg',
            'download_style': DownloadStyle.Direct.value,
        },
        'weights': {
            'url': 'https://github.com/AlexeyAB/darknet/releases/download/darknet_yolo_v3_optimal/yolov4.weights',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'yolov4',
            'size': 'weights(245.78 MB), cfg(11.94 KB)',
            'fps': '1.44 FPS'
        },
    },
    'yolov3': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (416, 416),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'cfg': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/yolov3.cfg',
            'download_style': DownloadStyle.Direct.value,
        },
        'weights': {
            'url': 'https://pjreddie.com/media/files/yolov3.weights',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'yolov3',
            'size': 'weights(236.52 MB), cfg(8.22 KB)',
            'fps': '3.24 FPS',
        },
    },
    'yolov3_ssp': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (608, 608),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'cfg': {
            'url': 'https://raw.githubusercontent.com/pjreddie/darknet/master/cfg/yolov3-spp.cfg',
            'download_style': DownloadStyle.Direct.value,
        },
        'weights': {
            'url': 'https://github.com/ultralytics/yolov3/releases/download/v8/yolov3-spp.weights',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'yolov3-spp',
            'size': 'weights(240.53 MB), cfg(8.4 KB)',
            'fps': '1.67 FPS',
            'info': 'TODO',
        },
    },
    'yolo_voc': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (416, 416),
        'scalefactor': 1 / 255,
        'mean': (127.5, 127.5, 127.5),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_20_LABELS,
        'cfg': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/yolo-voc.cfg',
            'download_style': DownloadStyle.Direct.value,
        },
        'weights': {
            'url': 'https://pjreddie.com/media/files/yolo-voc.weights',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'yolo-voc.weights',
            'size': 'weights(193.31 MB), cfg(2.66 KB)',
            'fps': '7.48 FPS',
        },
    },
    'MobileNetSSD_deploy': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (800, 600),
        'scalefactor': 1 / 127.5,
        'mean': (127.5, 127.5, 127.5),
        'swapRB': False,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_21_LABELS,
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/chuanqi305/MobileNet-SSD/' +
                   'daef68a6c2f5fbb8c88404266aa28180646d17e0/MobileNetSSD_deploy.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'https://drive.google.com/uc?export=download&id=0B3gersZ2cHIxRm5PMWRoTkdHdHc',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'MobileNetSSD_deploy',
            'size': 'caffemodel(22.08 MB), prototxt(28.67 KB)',
            'fps': '13.28 FPS',
            'info': 'https://www.programmersought.com/article/569030384/',
        },
    },
    'yolov3_tiny': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (512, 512),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_YOLO_80_LABELS,
        'cfg': {
            'url': 'https://raw.githubusercontent.com/pjreddie/darknet/master/cfg/yolov3-tiny.cfg',
            'download_style': DownloadStyle.Direct.value,
        },
        'weights': {
            'url': 'https://github.com/ultralytics/yolov3/releases/download/v8/yolov3-tiny.weights',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'yolov3-tiny.weights',
            'size': 'weights(33.79 MB), cfg(1.87 KB)',
            'fps': '18.56 FPS',
        },
    },
    'yolov2_tiny_voc': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Darknet.value,
        # YOLO FAMILY ALLOWED IN_DIMS: (320, 320), (416, 416), (512, 512), (608, 608)
        'in_dims': (416, 416),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_20_LABELS,
        'cfg': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/tiny-yolo-voc.cfg',
            'download_style': DownloadStyle.Direct.value,
        },
        'weights': {
            'url': 'https://pjreddie.com/media/files/yolov2-tiny-voc.weights',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'yolov2-tiny-voc.weights',
            'size': 'weights(60.53 MB), cfg(1.38 KB)',
            'fps': '24.61 FPS',
        },
    },
    'VGG_ILSVRC2016_SSD_300x300_deploy': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (300, 300),
        'scalefactor': 1,
        'mean': (127, 127, 127),
        'swapRB': False,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': ILSVRC2016_201_LABELS,
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/ssd_vgg16.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'https://www.dropbox.com/s/8apyk3uzk2vl522/' +
                   'VGG_ILSVRC2016_SSD_300x300_iter_440000.caffemodel?dl=1',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'VGG_ILSVRC2016_SSD_300x300_iter_440000.caffemodel',
            'size': 'caffemodel(192.06 MB), prototxt(23.93 KB)',
            'fps': '2.58',
        },
    },
    'VGG_ILSVRC_16_layers': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (102.9801, 115.9465, 122.7717),
        'swapRB': False,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_21_LABELS,
        'need_normalize': 'normalize output needed',
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'faster_rcnn_vgg16.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'https://dl.dropboxusercontent.com/s/o6ii098bu51d139/faster_rcnn_models.tgz?dl=0',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'VGG16_faster_rcnn_final.caffemodel',
        },
        'extra': {
            'web_name': 'VGG16_faster_rcnn_final.caffemodel',
            'size': 'caffemodel(522.92 MB), prototxt(8.45 KB)',
            'fps': '0.21 FPS',
        },
    },
    'rfcn_pascal_voc_resnet50': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (102.9801, 115.9465, 122.7717),
        'swapRB': False,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': PASCAL_VOC_2012_21_LABELS,
        'need_normalize': 'normalize output needed',
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'rfcn_pascal_voc_resnet50.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'https://onedrive.live.com/download?' +
                   'cid=10B28C0E28BF7B83&resid=10B28C0E28BF7B83%215317&authkey=%21AIeljruhoLuail8',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'resnet50_rfcn_final.caffemodel',
        },
        'extra': {
            'web_name': 'resnet50_rfcn_final.caffemodel',
            'size': 'caffemodel(121.58 MB), prototxt(62.8 KB)',
            'fps': '1.28 FPS',
        },
    },
    'faster_rcnn_zf': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (102.9801, 115.9465, 122.7717),
        'swapRB': False,
        'crop': False,
        'threshold': 0.4,
        'nms_threshold': 0.0,
        'labels_dict': PASCAL_VOC_2012_21_LABELS,
        'need_normalize': 'normalize output needed',
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'faster_rcnn_zf.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'https://dl.dropboxusercontent.com/s/o6ii098bu51d139/' +
                   'faster_rcnn_models.tgz?dl=0',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'ZF_faster_rcnn_final.caffemodel',
        },
        'extra': {
            'web_name': 'ZF_faster_rcnn_final',
            'size': 'caffemodel(226.19 MB), prototxt(7.1 KB)',
            'fps': '0.84 FPS',
        },
    },
    'ssd_inception_v2_coco_2017_11_17': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'pbtxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'ssd_inception_v2_coco_2017_11_17.pbtxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'pb': {
            'url': 'http://download.tensorflow.org/models/object_detection/ssd_inception_v2_coco_2017_11_17.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'frozen_inference_graph.pb',
        },
        'extra': {
            'web_name': 'ssd_inception_v2_coco_2017_11_17',
            'size': 'pb(97.26 MB), pbtxt(114.77 KB)',
            'fps': '3.78 FPS',
        },
    },
    'ssd_mobilenet_v1_coco': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (416, 416),
        'scalefactor': 1 / 127.5,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.4,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'pbtxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'ssd_mobilenet_v1_coco.pbtxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'pb': {
            'url': 'http://download.tensorflow.org/models/object_detection/ssd_mobilenet_v1_coco_11_06_2017.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'frozen_inference_graph.pb',
        },
        'extra': {
            'web_name': 'ssd_mobilenet_v1_coco_11_06_2017',
            'size': 'pb(27.76 MB), pbtxt(62.08 KB)',
            'fps': '27.13 FPS',
        },
    },
    'faster_rcnn_inception_v2_coco_2018_01_28': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.4,
        'nms_threshold': 0.4,
        'labels_dict': COCO_182_LABELS,
        'pbtxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'faster_rcnn_inception_v2_coco_2018_01_28.pbtxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'pb': {
            'url': 'http://download.tensorflow.org/models/object_detection/' +
                   'faster_rcnn_inception_v2_coco_2018_01_28.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'frozen_inference_graph.pb',
        },
        'extra': {
            'web_name': 'faster_rcnn_inception_v2_coco_2018_01_28',
            'size': 'pb(54.51 MB), pbtxt(112.92 KB)',
            'fps': '2.51 FPS',
        },
    },
    'faster_rcnn_resnet50_coco_2018_01_28': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_182_LABELS,
        'pbtxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'faster_rcnn_resnet50_coco_2018_01_28.pbtxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'pb': {
            'url': 'http://download.tensorflow.org/models/object_detection/faster_rcnn_resnet50_coco_2018_01_28.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'frozen_inference_graph.pb',
        },
        'extra': {
            'web_name': 'faster_rcnn_resnet50_coco_2018_01_28',
            'size': 'pb(114.97 MB), pbtxt(88.76 KB)',
            'fps': '0.84 FPS',
        },
    },
    'ssd_mobilenet_v1_coco_2017_11_17': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (300, 300),
        'scalefactor': 1,
        'mean': (0, 0, 0),
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'pbtxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'ssd_mobilenet_v1_coco_2017_11_17.pbtxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'pb': {
            'url': 'http://download.tensorflow.org/models/object_detection/ssd_mobilenet_v1_coco_2017_11_17.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'frozen_inference_graph.pb',
        },
        'extra': {
            'web_name': 'ssd_mobilenet_v1_coco_2017_11_17',
            'size': 'pb(27.76 MB), pbtxt(62.08 KB)',
            'fps': '46.28 FPS',
        },
    },
    'ssd_mobilenet_v1_ppn_coco': {  # checked
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (300, 300),
        'scalefactor': 1,
        'mean': (127.5, 127.5, 127.5),
        'swapRB': False,
        'crop': False,
        'threshold': 0.4,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,  # TODO labels could be wrong
        'pbtxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'ssd_mobilenet_v1_ppn_coco.pbtxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'pb': {
            'url': 'http://download.tensorflow.org/models/object_detection/' +
                   'ssd_mobilenet_v1_ppn_shared_box_predictor_300x300_coco14_sync_2018_07_03.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'frozen_inference_graph.pb',
        },
        'extra': {
            'web_name': 'ssd_mobilenet_v1_ppn_shared_box_predictor_300x300_coco14_sync_2018_07_03',
            'size': 'pb(10.29 MB), pbtxt(67.4 KB)',
            'fps': '38.72 FPS',
        },
    },
    'ssd_mobilenet_v2_coco_2018_03_29': {
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.TF.value,
        'in_dims': (800, 600),
        'scalefactor': 1,
        'mean': (0, 0, 0), 'file_to_look': 'frozen_inference_graph.caffemodel',
        'swapRB': True,
        'crop': False,
        'threshold': 0.2,
        'nms_threshold': 0.4,
        'labels_dict': COCO_183_LABELS,
        'pbtxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'ssd_mobilenet_v2_coco_2018_03_29.pbtxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'pb': {
            'url': 'http://download.tensorflow.org/models/object_detection/ssd_mobilenet_v2_coco_2018_03_29.tar.gz',
            'download_style': DownloadStyle.Tar.value,
            'file_to_look': 'frozen_inference_graph.pb',
        },
        'extra': {
            'web_name': 'ssd_mobilenet_v2_coco_2018_03_29',
            'size': 'pb(66.46 MB), pbtxt(112.89 KB)',
            'fps': '6.57 FPS',
        },
    },
    'opencv_face_detector': {
        'model_type': ModelType.OdCvNormal.value,
        'job': Jobs.OBJECT_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'in_dims': (520, 520),
        'scalefactor': 1,
        'mean': (104, 177, 123),
        'swapRB': False,
        'crop': False,
        'threshold': 0.5,
        'nms_threshold': None,
        'labels_dict': {
            'desc': 'TODO',
            'num_classes': 1,
            'labels': ['None', 'face']
        },
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/opencv/opencv_extra/master/testdata/dnn/' +
                   'opencv_face_detector.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'https://github.com/opencv/opencv_3rdparty/raw/dnn_samples_face_detector_20170830/' +
                   'res10_300x300_ssd_iter_140000.caffemodel',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'res10_300x300_ssd_iter_140000',
            'size': 'caffemodel(10.17 MB), prototxt(27.45 KB)',
            'info1': 'https://www.programmersought.com/article/16544476883/',
            'info2': 'https://www.programmersought.com/article/16544476883/',
            'info3': 'https://www.pyimagesearch.com/2018/02/26/face-detection-with-opencv-and-deep-learning/',
        },
    },
    # pose detection TFL and cv2
    # TFL
    'pose_landmark_full': {
        'model_type': ModelType.PdTflNormal.value,
        'job': Jobs.POSE_DETECTION.value,
        'threshold': 0.2,
        'joint_names': {
            0: 'nose', 1: 'leftEyeInside', 2: 'leftEye', 3: 'leftEyeOutside', 4: 'rightEyeInside',
            5: 'rightEye', 6: 'rightEyeOutside', 7: 'leftEar', 8: 'rightEar', 9: 'leftMouth',
            10: 'rightMouth', 11: 'leftShoulder', 12: 'rightShoulder', 13: 'leftElbow', 14: 'rightElbow',
            15: 'leftWrist', 16: 'rightWrist', 17: 'leftPalm', 18: 'rightPalm', 19: 'leftIndex',
            20: 'rightIndex', 21: 'leftPinky', 22: 'rightPinky', 23: 'leftHip', 24: 'rightHip',
            25: 'leftKnee', 26: 'rightKnee', 27: 'leftAnkle', 28: 'rightAnkle', 29: 'leftHeel',
            30: 'rightHeel', 31: 'leftFoot', 32: 'rightFoot'
            # don't think those are supported
            # 33: 'midHip', 34: 'forehead',35: 'leftThumb',36: 'leftHand',37: 'rightThumb',38: 'rightHand',
        },
        'joint_colors': [
            'red', 'magenta', 'lime', 'magenta', 'magenta',
            'lime', 'magenta', 'lime', 'lime', 'lime',
            'lime', 'darkorange', 'darkorange', 'yellow', 'yellow',
            'darkorange', 'darkorange', 'aqua', 'aqua', 'lime',
            'lime', 'yellow', 'yellow', 'aqua', 'aqua',
            'magenta', 'magenta', 'aqua', 'aqua', 'magenta',
            'magenta', 'aqua', 'aqua',
        ],
        'pairs_indices': [
            [7, 3], [3, 2], [2, 1], [1, 0], [0, 9], [9, 10], [10, 0], [0, 4], [4, 5], [5, 6], [6, 8],
            [17, 15], [15, 13], [13, 11], [18, 16], [16, 14], [14, 12], [23, 25], [25, 27], [27, 29],
            [29, 31], [24, 26], [26, 28], [28, 30], [30, 32], [23, 24], [24, 12], [12, 11], [11, 23],
        ],
        'pairs_indices_colors': [
            'lime', 'lime', 'lime', 'lime', 'lime',
            'lime', 'lime', 'lime', 'lime', 'lime',
            'lime', 'yellow', 'darkorange', 'yellow', 'yellow',
            'darkorange', 'yellow', 'aqua', 'magenta', 'aqua',
            'magenta', 'aqua', 'magenta', 'aqua', 'magenta',
            'aqua', 'lime', 'darkorange', 'lime'
        ],
        'tflite': {
            'url': 'https://github.com/google/mediapipe/raw/master/mediapipe/modules/pose_landmark/' +
                   'pose_landmark_full.tflite',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'pose_landmark_full',
            'size': 'tflite(6.13 MB)',
            'fps': 'TODO',
            'info': 'https://github.com/google/mediapipe/tree/master/mediapipe/modules/pose_detection',
            'info2': 'https://github.com/vladmandic/blazepose/tree/fe647445507e37469d96da6fde5c8b0980f745bc'
        },
    },
    'pose_landmark_lite': {
        'model_type': ModelType.PdTflNormal.value,
        'job': Jobs.POSE_DETECTION.value,
        'threshold': 0.2,
        'joint_names': {
            0: 'nose', 1: 'leftEyeInside', 2: 'leftEye', 3: 'leftEyeOutside', 4: 'rightEyeInside',
            5: 'rightEye', 6: 'rightEyeOutside', 7: 'leftEar', 8: 'rightEar', 9: 'leftMouth',
            10: 'rightMouth', 11: 'leftShoulder', 12: 'rightShoulder', 13: 'leftElbow', 14: 'rightElbow',
            15: 'leftWrist', 16: 'rightWrist', 17: 'leftPalm', 18: 'rightPalm', 19: 'leftIndex',
            20: 'rightIndex', 21: 'leftPinky', 22: 'rightPinky', 23: 'leftHip', 24: 'rightHip',
            25: 'leftKnee', 26: 'rightKnee', 27: 'leftAnkle', 28: 'rightAnkle', 29: 'leftHeel',
            30: 'rightHeel', 31: 'leftFoot', 32: 'rightFoot'
            # don't think those are supported
            # 33: 'midHip', 34: 'forehead',35: 'leftThumb',36: 'leftHand',37: 'rightThumb',38: 'rightHand',
        },
        'joint_colors': [
            'red', 'magenta', 'lime', 'magenta', 'magenta',
            'lime', 'magenta', 'lime', 'lime', 'lime',
            'lime', 'darkorange', 'darkorange', 'yellow', 'yellow',
            'darkorange', 'darkorange', 'aqua', 'aqua', 'lime',
            'lime', 'yellow', 'yellow', 'aqua', 'aqua',
            'magenta', 'magenta', 'aqua', 'aqua', 'magenta',
            'magenta', 'aqua', 'aqua',
        ],
        'pairs_indices_colors': [
            'lime', 'lime', 'lime', 'lime', 'lime',
            'lime', 'lime', 'lime', 'lime', 'lime',
            'lime', 'yellow', 'darkorange', 'yellow', 'yellow',
            'darkorange', 'yellow', 'aqua', 'magenta', 'aqua',
            'magenta', 'aqua', 'magenta', 'aqua', 'magenta',
            'aqua', 'lime', 'darkorange', 'lime'
        ],
        'pairs_indices': [
            [7, 3], [3, 2], [2, 1], [1, 0], [0, 9],
            [9, 10], [10, 0], [0, 4], [4, 5], [5, 6],
            [6, 8], [17, 15], [15, 13], [13, 11], [18, 16],
            [16, 14], [14, 12], [23, 25], [25, 27], [27, 29],
            [29, 31], [24, 26], [26, 28], [28, 30], [30, 32],
            [23, 24], [24, 12], [12, 11], [11, 23],
        ],
        'tflite': {
            'url': 'https://github.com/google/mediapipe/raw/master/mediapipe/modules/pose_landmark/' +
                   'pose_landmark_lite.tflite',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'TODO',
            'size': 'tflite(2.68 MB)',
            'fps': 'TODO',
            'info': 'https://github.com/google/mediapipe/tree/master/mediapipe/modules/pose_landmark',
            'info2': 'https://github.com/vladmandic/blazepose/tree/fe647445507e37469d96da6fde5c8b0980f745bc'
        },
    },
    'pose_landmark_heavy': {
        'model_type': ModelType.PdTflNormal.value,
        'job': Jobs.POSE_DETECTION.value,
        'threshold': 0.2,
        'joint_names': {
            0: 'nose', 1: 'leftEyeInside', 2: 'leftEye', 3: 'leftEyeOutside', 4: 'rightEyeInside',
            5: 'rightEye', 6: 'rightEyeOutside', 7: 'leftEar', 8: 'rightEar', 9: 'leftMouth',
            10: 'rightMouth', 11: 'leftShoulder', 12: 'rightShoulder', 13: 'leftElbow', 14: 'rightElbow',
            15: 'leftWrist', 16: 'rightWrist', 17: 'leftPalm', 18: 'rightPalm', 19: 'leftIndex',
            20: 'rightIndex', 21: 'leftPinky', 22: 'rightPinky', 23: 'leftHip', 24: 'rightHip',
            25: 'leftKnee', 26: 'rightKnee', 27: 'leftAnkle', 28: 'rightAnkle', 29: 'leftHeel',
            30: 'rightHeel', 31: 'leftFoot', 32: 'rightFoot'
            # don't think those are supported
            # 33: 'midHip', 34: 'forehead',35: 'leftThumb',36: 'leftHand',37: 'rightThumb',38: 'rightHand',
        },
        'joint_colors': [
            'red', 'magenta', 'lime', 'magenta', 'magenta',
            'lime', 'magenta', 'lime', 'lime', 'lime',
            'lime', 'darkorange', 'darkorange', 'yellow', 'yellow',
            'darkorange', 'darkorange', 'aqua', 'aqua', 'lime',
            'lime', 'yellow', 'yellow', 'aqua', 'aqua',
            'magenta', 'magenta', 'aqua', 'aqua', 'magenta',
            'magenta', 'aqua', 'aqua',
        ],
        'pairs_indices': [
            [7, 3], [3, 2], [2, 1], [1, 0], [0, 9], [9, 10], [10, 0], [0, 4], [4, 5], [5, 6], [6, 8],
            [17, 15], [15, 13], [13, 11], [18, 16], [16, 14], [14, 12], [23, 25], [25, 27], [27, 29],
            [29, 31], [24, 26], [26, 28], [28, 30], [30, 32], [23, 24], [24, 12], [12, 11], [11, 23],
        ],
        'pairs_indices_colors': [
            'lime', 'lime', 'lime', 'lime', 'lime',
            'lime', 'lime', 'lime', 'lime', 'lime',
            'lime', 'yellow', 'darkorange', 'yellow', 'yellow',
            'darkorange', 'yellow', 'aqua', 'magenta', 'aqua',
            'magenta', 'aqua', 'magenta', 'aqua', 'magenta',
            'aqua', 'lime', 'darkorange', 'lime'
        ],
        'tflite': {
            'url': 'https://github.com/google/mediapipe/raw/master/mediapipe/modules/pose_landmark/' +
                   'pose_landmark_heavy.tflite',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'pose_landmark_heavy',
            'size': 'tflite(26.42 MB)',
            'fps': 'TODO',
            'info': 'https://github.com/google/mediapipe/tree/master/mediapipe/modules/pose_detection',
            'info2': 'https://github.com/vladmandic/blazepose/tree/fe647445507e37469d96da6fde5c8b0980f745bc'
        },
    },
    'posenet': {
        'model_type': ModelType.PdTflPoseNet.value,
        'job': Jobs.POSE_DETECTION.value,
        'threshold': 0.2,
        'joint_names': {
            0: 'nose', 1: 'leftEye', 2: 'rightEye', 3: 'leftEar', 4: 'rightEar',
            5: 'leftShoulder', 6: 'rightShoulder', 7: 'leftElbow', 8: 'rightElbow', 9: 'leftWrist',
            10: 'rightWrist', 11: 'leftHip', 12: 'rightHip', 13: 'leftKnee', 14: 'rightKnee',
            15: 'leftAnkle', 16: 'rightAnkle',
        },
        'joint_colors': [
            'red', 'magenta', 'magenta', 'lime', 'lime',
            'darkorange', 'darkorange', 'yellow', 'yellow', 'darkorange',
            'darkorange', 'aqua', 'aqua', 'magenta', 'magenta',
            'aqua', 'aqua'
        ],
        'pairs_indices': [
            [5, 6], [5, 7], [7, 9], [5, 11], [6, 8],
            [8, 10], [6, 12], [11, 12], [11, 13], [13, 15],
            [12, 14], [14, 16]
        ],
        'pairs_indices_colors': [
            'darkorange', 'yellow', 'darkorange', 'lime', 'yellow',
            'darkorange', 'lime', 'aqua', 'aqua', 'magenta',
            'aqua', 'magenta'
        ],
        'tflite': {
            'url': 'https://storage.googleapis.com/download.tensorflow.org/models/tflite/' +
                   'posenet_mobilenet_v1_100_257x257_multi_kpt_stripped.tflite',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'posenet',
            'size': 'tflite(12.65 MB)',
            'fps': 'TODO',
            'info': 'https://github.com/ecd1012/rpi_pose_estimation/blob/main/run_pose_estimation.py',
            'info2': 'https://www.tensorflow.org/lite/examples/pose_estimation/overview',
        },
    },
    # cv2
    'openpose_pose_mpi': {
        'model_type': ModelType.PdCvNormal.value,
        'job': Jobs.POSE_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'joint_names': {
            0: 'Head', 1: 'Neck', 2: 'RShoulder', 3: 'RElbow', 4: 'RWrist',
            5: 'LShoulder', 6: 'LElbow', 7: 'LWrist', 8: 'RHip', 9: 'RKnee',
            10: 'RAnkle', 11: 'LHip', 12: 'LKnee', 13: 'LAnkle', 14: 'Chest',
            15: 'Background'
        },
        'joint_colors': [
            'darkorange', 'red', 'darkorange', 'yellow', 'darkorange',
            'darkorange', 'yellow', 'darkorange', 'lime', 'aqua',
            'magenta', 'lime', 'aqua', 'magenta', 'aqua',
            'black'
        ],
        'pairs_indices': [
            [0, 1], [1, 2], [2, 3], [3, 4], [1, 5],
            [5, 6], [6, 7], [1, 14], [14, 8], [8, 9],
            [9, 10], [14, 11], [11, 12], [12, 13]
        ],
        'pairs_indices_colors': [
            'red', 'darkorange', 'yellow', 'darkorange', 'darkorange',
            'yellow', 'darkorange', 'aqua', 'lime', 'aqua',
            'magenta', 'lime', 'aqua', 'magenta'
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/openpose/master/models/' +
                   'pose/mpi/pose_deploy_linevec_faster_4_stages.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/pose/mpi/pose_iter_160000.caffemodel',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'pose_iter_160000',
            'size': 'caffemodel(196.41 MB), prototxt(45.83 KB)',
            'fps': 'TODO',
            'info': 'https://www.programmersought.com/article/3282857837/',
        },
    },
    'openpose_pose_coco': {
        'model_type': ModelType.PdCvNormal.value,
        'job': Jobs.POSE_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'joint_names': {
            0: 'Nose', 1: 'Neck', 2: 'RShoulder', 3: 'RElbow', 4: 'RWrist',
            5: 'LShoulder', 6: 'LElbow', 7: 'LWrist', 8: 'RHip', 9: 'RKnee',
            10: 'RAnkle', 11: 'LHip', 12: 'LKnee', 13: 'LAnkle', 14: 'REye',
            15: 'LEye', 16: 'REar', 17: 'LEar', 18: 'Background',
        },
        'joint_colors': [
            'red', 'red', 'darkorange', 'yellow', 'darkorange',
            'darkorange', 'yellow', 'darkorange', 'lime', 'aqua',
            'magenta', 'lime', 'aqua', 'magenta', 'blue',
            'blue', 'aqua', 'aqua', 'black'
        ],
        'pairs_indices': [
            [1, 2], [1, 5], [2, 3], [3, 4], [5, 6],
            [6, 7], [1, 8], [8, 9], [9, 10], [1, 11],
            [11, 12], [12, 13], [1, 0], [0, 14], [14, 16],
            [0, 15], [15, 17]
        ],
        'pairs_indices_colors': [
            'darkorange', 'darkorange', 'yellow', 'darkorange', 'yellow',
            'darkorange', 'lime', 'aqua', 'magenta', 'lime',
            'aqua', 'magenta', 'red', 'blue', 'aqua',
            'blue', 'aqua'
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/openpose/master/models/pose/' +
                   'coco/pose_deploy_linevec.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/pose/coco/pose_iter_440000.caffemodel',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'pose_iter_440000',
            'size': 'caffemodel(199.58 MB), prototxt(45.78 KB)',
            'fps': 'TODO',
            'info1': 'https://github.com/CMU-Perceptual-Computing-Lab/openpose',
            'info2': 'https://www.programmersought.com/article/3282857837/',
        },
    },
    'openpose_pose_coco_multi': {
        'model_type': ModelType.PdCvCocoMulti.value,
        'job': Jobs.POSE_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'joint_names': {
            0: 'Nose', 1: 'Neck', 2: 'RShoulder', 3: 'RElbow', 4: 'RWrist',
            5: 'LShoulder', 6: 'LElbow', 7: 'LWrist', 8: 'RHip', 9: 'RKnee',
            10: 'RAnkle', 11: 'LHip', 12: 'LKnee', 13: 'LAnkle', 14: 'REye',
            15: 'LEye', 16: 'REar', 17: 'LEar'
        },
        'joint_colors': [
            'red', 'red', 'darkorange', 'yellow', 'darkorange',
            'darkorange', 'yellow', 'darkorange', 'lime', 'aqua',
            'magenta', 'lime', 'aqua', 'magenta', 'blue',
            'blue', 'aqua', 'aqua'
        ],
        'pairs_indices': [
            [1, 2], [1, 5], [2, 3], [3, 4], [5, 6],
            [6, 7], [1, 8], [8, 9], [9, 10], [1, 11],
            [11, 12], [12, 13], [1, 0], [0, 14], [14, 16],
            [0, 15], [15, 17]
        ],
        'pairs_indices_colors': [
            'darkorange', 'darkorange', 'yellow', 'darkorange', 'yellow',
            'darkorange', 'lime', 'aqua', 'magenta', 'lime',
            'aqua', 'magenta', 'red', 'blue', 'aqua',
            'blue', 'aqua'
        ],
        'pafs_indices': [
            # index of pafs corresponding to the POSE_PAIRS
            # e.g for POSE_PAIR(1,2), the PAFs are located at indices (31,32) of output,
            # Similarly, (1,5) -> (39,40) and so on.
            [31, 32], [39, 40], [33, 34], [35, 36], [41, 42],
            [43, 44], [19, 20], [21, 22], [23, 24], [25, 26],
            [27, 28], [29, 30], [47, 48], [49, 50], [53, 54],
            [51, 52], [55, 56]
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/openpose/master/models/pose/' +
                   'coco/pose_deploy_linevec.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/pose/coco/pose_iter_440000.caffemodel',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'pose_iter_440000',
            'size': 'caffemodel(199.58 MB), prototxt(45.78 KB)',
            'fps': 'TODO',
            'info1': 'https://github.com/CMU-Perceptual-Computing-Lab/openpose',
            'info2': 'https://www.programmersought.com/article/3282857837/',
            'info3': 'https://github.com/spmallick/learnopencv/blob/master/OpenPose-Multi-Person/' +
                     'multi-person-openpose.py',
        },
    },
    'body25': {
        'model_type': ModelType.PdCvNormal.value,
        'job': Jobs.POSE_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'joint_names': {
            0: 'Nose', 1: 'Neck', 2: 'RShoulder', 3: 'RElbow', 4: 'RWrist',
            5: 'LShoulder', 6: 'LElbow', 7: 'LWrist', 8: 'MidHip', 9: 'RHip',
            10: 'RKnee', 11: 'RAnkle', 12: 'LHip', 13: 'LKnee', 14: 'LAnkle',
            15: 'REye', 16: 'LEye', 17: 'REar', 18: 'LEar', 19: 'LBigToe',
            20: 'LSmallToe', 21: 'LHeel', 22: 'RBigToe', 23: 'RSmallToe', 24: 'RHeel',
            25: 'Background',
        },
        'joint_colors': [  # TODO fix on a clearer picture
            'red', 'red', 'darkorange', 'yellow', 'darkorange',
            'darkorange', 'yellow', 'darkorange', 'lime', 'aqua',
            'magenta', 'aqua', 'aqua', 'magenta', 'aqua',
            'lime', 'lime', 'magenta', 'magenta', 'blue',
            'magenta', 'lime', 'aqua', 'magenta', 'blue',
            'blue',
        ],
        'pairs_indices': [
            [1, 0], [1, 2], [1, 5], [2, 3], [3, 4],
            [5, 6], [6, 7], [0, 15], [15, 17], [0, 16],
            [16, 18], [1, 8], [8, 9], [9, 10], [10, 11],
            [11, 22], [22, 23], [11, 24], [8, 12], [12, 13],
            [13, 14], [14, 19], [19, 20], [14, 21]
        ],
        'pairs_indices_colors': [
            'red', 'darkorange', 'darkorange', 'yellow', 'darkorange',
            'yellow', 'darkorange', 'lime', 'lime', 'lime',
            'lime', 'lime', 'lime', 'aqua', 'magenta',
            'aqua', 'lime', 'darkorange', 'lime', 'aqua',
            'magenta', 'aqua', 'lime', 'darkorange'
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/' +
                   'openpose/master/models/pose/body_25/pose_deploy.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/pose/body_25/' +
                   'pose_iter_584000.caffemodel',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'pose_iter_584000.caffemodel',
            'size': 'caffemodel(99.86 MB), prototxt(41.34 KB)',
            'fps': 'TODO',
            'info1': 'https://github.com/CMU-Perceptual-Computing-Lab/openpose',
            'info2': 'https://www.programmersought.com/article/3282857837/',
            'info3': 'https://cmu-perceptual-computing-lab.github.io/openpose/web/html/doc/md_doc_02_output.html',
        },
    },
    'hand_pose': {
        'model_type': ModelType.PdCvNormal.value,
        'job': Jobs.POSE_DETECTION.value,
        'family': DnnFamily.Caffe.value,
        'joint_names': {
            0: 'handRoot',
            1: 'thumbBase', 2: 'thumbMid1', 3: 'thumbMid2', 4: 'thumbNail',
            5: 'indexBase', 6: 'indexMid1', 7: 'indexMid2', 8: 'indexNail',
            9: 'middleBase', 10: 'middleMid1', 11: 'middleMid2', 12: 'middleNail',
            13: 'ringBase', 14: 'ringMid1', 15: 'ringMid2', 16: 'ringNail',
            17: 'pinkyBase', 18: 'pinkyMid1', 19: 'pinkyMid2', 20: 'pinkyNail',
        },
        'joint_colors': [
            'red',
            'darkorange', 'aqua', 'lime', 'yellow',
            'darkorange', 'aqua', 'lime', 'yellow',
            'darkorange', 'aqua', 'lime', 'yellow',
            'darkorange', 'aqua', 'lime', 'yellow',
            'darkorange', 'aqua', 'lime', 'yellow',
        ],
        'pairs_indices': [
            [0, 1], [1, 2], [2, 3], [3, 4],  # thumb
            [0, 5], [5, 6], [6, 7], [7, 8],  # index
            [0, 9], [9, 10], [10, 11], [11, 12],  # middle
            [0, 13], [13, 14], [14, 15], [15, 16],  # ring
            [0, 17], [17, 18], [18, 19], [19, 20]  # pinky
        ],
        'pairs_indices_colors': [
            'darkorange', 'aqua', 'lime', 'yellow',
            'darkorange', 'aqua', 'lime', 'yellow',
            'darkorange', 'aqua', 'lime', 'yellow',
            'darkorange', 'aqua', 'lime', 'yellow',
            'darkorange', 'aqua', 'lime', 'yellow'
        ],
        'in_dims': (368, 368),
        'scalefactor': 1 / 255,
        'mean': (0, 0, 0),
        'swapRB': False,
        'crop': False,
        'threshold': 0.1,
        'prototxt': {
            'url': 'https://raw.githubusercontent.com/CMU-Perceptual-Computing-Lab/openpose/master/models/' +
                   'hand/pose_deploy.prototxt',
            'download_style': DownloadStyle.Direct.value,
        },
        'caffemodel': {
            'url': 'http://posefs1.perception.cs.cmu.edu/OpenPose/models/hand/pose_iter_102000.caffemodel',
            'download_style': DownloadStyle.Direct.value,
        },
        'extra': {
            'web_name': 'pose_iter_102000.caffemodel',
            'size': 'caffemodel(140.52 MB), prototxt(25.83 KB)',
            'fps': 'TODO',
            'info1': 'https://www.programmersought.com/article/1536716931/',
        },
    },
}



from wizzi_utils.misc import misc_tools as mt
import numpy as np
import os

PLAY_GROUND = './wizzi_utils_playground'

IMAGES_PATH = '{}/images'.format(PLAY_GROUND)
IMAGES_INPUTS = '{}/Input'.format(IMAGES_PATH)
IMAGES_OUTPUTS = '{}/Output'.format(IMAGES_PATH)

VIDEOS_PATH = '{}/videos'.format(PLAY_GROUND)
VIDEOS_INPUTS = '{}/Input'.format(VIDEOS_PATH)
VIDEOS_OUTPUTS = '{}/Output'.format(VIDEOS_PATH)

MODELS = '{}/models'.format(PLAY_GROUND)
DATASETS = '{}/datasets'.format(PLAY_GROUND)
TEMP_FOLDER1 = '{}/temp_folder1'.format(PLAY_GROUND)
TEMP_FOLDER2 = '{}/temp_folder2'.format(PLAY_GROUND)

DEMO_FILE = 'demo_file.txt'
JUST_A_NAME = 'just_a_name'
FAKE_FILE = 'fake_file'

SO_LOGO = 'so_logo'
KITE = 'kite'
GIRAFFE = 'giraffe'
HORSES = 'horses'
DOG = 'dog'
EAGLE = 'eagle'
PERSON = 'person'
DOGS1 = 'dogs1'
FACES = 'faces'
F_MODEL = 'model2'
HAND = 'hand'

# sources: https://github.com/pjreddie/darknet/blob/master/data/
IMAGES_D = {
    SO_LOGO: 'https://cdn.sstatic.net/Sites/stackoverflow/img/logo.png',
    KITE: 'https://raw.githubusercontent.com/pjreddie/darknet/master/data/kite.jpg',
    GIRAFFE: 'https://raw.githubusercontent.com/pjreddie/darknet/master/data/giraffe.jpg',
    HORSES: 'https://raw.githubusercontent.com/pjreddie/darknet/master/data/horses.jpg',
    DOG: 'https://raw.githubusercontent.com/pjreddie/darknet/master/data/dog.jpg',
    EAGLE: 'https://raw.githubusercontent.com/pjreddie/darknet/master/data/eagle.jpg',
    PERSON: 'https://raw.githubusercontent.com/pjreddie/darknet/master/data/person.jpg',
    DOGS1: 'https://media.blogto.com/articles/20210415-dog-parks-toronto.jpg' +
           '?w=1200&cmd=resize_then_crop&height=630&quality=70',
    FACES: 'https://upload.wikimedia.org/wikipedia/commons/thumb/f/f8/' +
           'Young_People_in_Park_-_Stepanakert_-_Nagorno-Karabakh_%2819082779012%29.jpg/' +
           '640px-Young_People_in_Park_-_Stepanakert_-_Nagorno-Karabakh_%2819082779012%29.jpg',
    # F_L: 'https://github.com/vladmandic/blazepose/raw/fe647445507e37469d96da6fde5c8b0980f745bc/outputs/model2.jpg',
    F_MODEL: 'https://github.com/vladmandic/blazepose/raw/fe647445507e37469d96da6fde5c8b0980f745bc/inputs/model2.jpg',
    HAND: 'http://clipart-library.com/images/8TxrGaBgc.jpg',
}

DOG1 = 'dog_in_a_field'
WOMAN_YOGA = 'woman_yoga'
VIDEOS_D = {  # https://www.shutterstock.com/video
    DOG1: 'https://ak.picdn.net/shutterstock/videos/31512310/preview/' +
          'stock-footage-curious-beagle-dog-run-at-grass-chase-moving-camera' +
          '-slow-motion-shot-long-ears-flap-and-fly-in.webm',
    WOMAN_YOGA: 'https://media.istockphoto.com/videos/your-body-knows-what-it-needs-video-id1161189526'
}


def create_demo_file(path: str, tabs: int = 1) -> None:
    if not os.path.exists(os.path.dirname(path)):
        mt.create_dir(os.path.dirname(path))
    f = open(path, "w")
    f.write("Now the file has more content!\n" * 1000)
    f.close()
    file_msg = '{}({})'.format(path, mt.file_or_folder_size(path))
    print('{}{}'.format(tabs * '\t', mt.CREATED.format(file_msg)))
    return


def timer_test():
    mt.get_function_name(ack=True, tabs=0)
    start_t = mt.get_timer()
    total = mt.get_timer_delta(s_timer=start_t, with_ms=True)
    total_full = mt.get_timer_delta(s_timer=start_t, with_ms=False)
    print('\tTotal run time {}'.format(total))
    print('\tTotal run time {}'.format(total_full))
    mt.get_timer_delta(s_timer=start_t, with_ms=False, ack=True, tabs=1)
    return


def timer_action_test():
    mt.get_function_name(ack=True, tabs=0)
    print('\ttimer count down example:')
    for i in range(1):
        mt.timer_action(seconds=2, action='take image {}'.format(i), tabs=2)
    # TODO restore after fixing open cv start minimized
    # print('\t press key example:')
    # mt.timer_action(seconds=None, action='taking an image', tabs=2)
    return


def get_time_stamp_test():
    mt.get_function_name(ack=True, tabs=0)
    print('\tdate no day: {}'.format(mt.get_time_stamp(format_s='%Y_%m')))
    print('\tdate: {}'.format(mt.get_time_stamp(format_s='%Y_%m_%d')))
    print('\ttime: {}'.format(mt.get_time_stamp(format_s='%H_%M_%S')))
    mt.get_time_stamp(ack=True, tabs=1)
    print('\ttime stamp for files: {}'.format(mt.get_time_stamp(format_s='%Y_%m_%d_%H_%M_%S')))
    print('\tdate and time with ms: {}'.format(mt.get_time_stamp(format_s='%Y_%m_%d_%H_%M_%S.%f')))
    print('\tdate Israel format {}'.format(mt.get_time_stamp(format_s='%d-%m-%Y %H:%M:%S')))
    return


def get_pc_name_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_pc_name(ack=True, tabs=1)
    print('\tPc name is {}'.format(mt.get_pc_name()))
    return


def get_mac_address_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_mac_address(ack=True, tabs=1)
    mt.get_mac_address(with_colons=False, ack=True, tabs=1)
    print('\tMac address is {}'.format(mt.get_mac_address()))
    return


def get_cuda_version_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_cuda_version(ack=True, tabs=1)
    return


def get_env_variables_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_env_variables(ack=True, tabs=1)
    return


def set_env_variable_test():
    mt.get_function_name(ack=True, tabs=0)
    k = JUST_A_NAME
    mt.set_env_variable(key=k, val=JUST_A_NAME, ack=True, tabs=1)
    print('\tCheck:')
    mt.get_env_variable(key=k, ack=True, tabs=2)
    mt.del_env_variable(key=k, ack=True, tabs=1)
    return


def get_env_variable_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_env_variable(key=JUST_A_NAME, ack=True, tabs=1)
    mt.get_env_variable(key='PATH', ack=True, tabs=1)
    return


def del_env_variable_test():
    mt.get_function_name(ack=True, tabs=0)
    k = JUST_A_NAME
    mt.set_env_variable(key=k, val=JUST_A_NAME, ack=True, tabs=1)

    mt.del_env_variable(key=k, ack=True, tabs=1)
    mt.del_env_variable(key='v2_{}'.format(JUST_A_NAME), ack=True, tabs=1)
    return


def make_cuda_invisible_test():
    mt.get_function_name(ack=True, tabs=0)
    k = 'CUDA_VISIBLE_DEVICES'

    old_value = mt.get_env_variable(key=k)  # save old value
    mt.make_cuda_invisible()  # change to new value
    new_value = mt.get_env_variable(key=k)  # try get new value

    if new_value is not None:
        print('\t{} = {}'.format(k, new_value))
    else:
        print('\tTest Failed')

    # restore old value
    if old_value is None:
        mt.del_env_variable(key=k)  # didn't exist: delete it
    else:
        mt.set_env_variable(key=k, val=old_value)  # existed: restore value

    return


def profiler_test():
    mt.get_function_name(ack=True, tabs=0)
    pr = mt.start_profiler()
    mt.get_function_name(ack=False)
    profiler_str = mt.end_profiler(pr, rows=5, ack=True)
    print(profiler_str)
    return


def os_test():
    mt.get_function_name(ack=True, tabs=0)
    print('\tis_windows     {}'.format(mt.is_windows()))
    print('\tis_linux       {}'.format(mt.is_linux()))

    print('\tis_armv7l      {}'.format(mt.is_armv7l()))
    print('\tis_raspberrypi {}'.format(mt.is_raspberry_pi()))

    print('\tis_aarch64     {}'.format(mt.is_aarch64()))
    print('\tis_jetson_nano {}'.format(mt.is_jetson_nano()))
    return


def get_wizzi_utils_root_test():
    mt.get_function_name(ack=True, tabs=0)
    print('\t{}'.format(mt.get_wizzi_utils_root()))
    return


def main_wrapper_test():
    def temp_function():
        print('hello_world')

    mt.get_function_name(ack=True, tabs=0)
    mt.main_wrapper(
        main_function=temp_function,
        seed=42,
        ipv4=True,
        cuda_off=False,
        torch_v=True,
        tf_v=True,
        cv2_v=True,
        with_profiler=False
    )
    return


def to_str_test():
    mt.get_function_name(ack=True, tabs=0)

    # INTS
    x = 1234
    print('\t{}'.format(mt.to_str(var=x)))  # minimal

    x = 12345678912345
    print(mt.to_str(var=x, title='\tvery long int'))

    # FLOATS
    f = 3.4
    print(mt.to_str(var=f, title='\tsmall float', fp=-1))

    f = 3.2123123
    print(mt.to_str(var=f, title='\tlong float(rounded 4 digits)', fp=4))

    f = 1234567890.223123123123123123
    print(mt.to_str(var=f, title='\tbig long float(rounded 3 digits)', fp=3))

    # STRINGS
    s = 'hello world'
    print(mt.to_str(var=s, title='\tregular string'))

    s = ''
    print(mt.to_str(var=s, title='\tempty string'))

    # LISTS
    li = []
    print(mt.to_str(var=li, title='\tempty list'))

    li = [112312312, 3, 4]
    print(mt.to_str(var=li, title='\tlist of ints(recursive print)', rec=True))

    li = [112312312, 3, 4]
    print(mt.to_str(var=li, title='\tlist of ints(no metadata)', wm=False))

    li = [1, 3123123]
    print(mt.to_str(var=li, title='\t1d list of ints(no data)', chars=0))
    print(mt.to_str(var=li, title='\t1d list of ints(all data)', chars=-1))

    li = [1.0000012323, 3123123.22454875123123]
    print(mt.to_str(var=li, title='\t1d list(rounded 7 digits)', fp=7))

    li = [12, 1.2323, 'HI']
    print(mt.to_str(var=li, title='\t1d mixed list'))

    li = [4e-05, 6e-05, 4e-05, 5e-05, 4e-05, 7e-05, 2e-05, 7e-05, 5e-05, 8e-05]
    print(mt.to_str(var=li, title='\te-0 style', fp=6))

    li = [[4e-05, 6e-05, 4e-05, 5e-05, 4e-05, 7e-05, 2e-05, 7e-05, 5e-05, 8e-05]]
    print(mt.to_str(var=li, title='\t2d e-0 style', fp=6, rec=True))

    li = [11235] * 1000
    print(mt.to_str(var=li, title='\t1d long list'))

    li = [[1231.2123123, 15.9], [3.0, 7.55]]
    print(mt.to_str(var=li, title='\t2d list', rec=True))

    li = [(1231.2123123, 15.9), (3.0, 7.55)]
    print(mt.to_str(var=li, title='\t2d list of tuples', rec=True))

    # TUPLES
    t = (1239.123123, 3.12, 9.123123123123)
    print(mt.to_str(var=t, title='\t1d tuple', rec=True))

    # NUMPY
    ni = np.array([4e-05, 6e-02], dtype=float)
    print(mt.to_str(var=ni, title='\t1d np array(fp=5)', fp=5, rec=False))

    ni = np.array([1.0000012323, 3123123.22454875123123], dtype=float)
    print(mt.to_str(var=ni, title='\t1d np array(fp=2)', fp=2, rec=False))

    ni = np.array([[1231.123122, 15.9], [3.0, 7.55], [4e-05, 6e-02]])
    print(mt.to_str(var=ni, title='\t2d np array', rec=True))

    cv_img = np.zeros(shape=[480, 640, 3], dtype=np.uint8)
    print(mt.to_str(var=cv_img, title='\tcv_img', chars=20))

    # DICTS
    di = {'a': [1213, 2]}
    print(mt.to_str(var=di, title='\tdict of lists', rec=True))

    di = {'a': [{'k': [1, 2]}, {'c': [7, 2]}]}
    print(mt.to_str(var=di, title='\tnested dict', rec=True))
    return


def save_load_np_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.create_dir(TEMP_FOLDER1)
    path = '{}/{}.npy'.format(TEMP_FOLDER1, JUST_A_NAME)
    a = np.ones(shape=(2, 3, 29))
    print(mt.to_str(a, '\ta'))
    mt.save_np(a, path=path)
    a2 = mt.load_np(path, ack=True)
    print(mt.to_str(a2, '\ta2'))
    mt.delete_dir_with_files(TEMP_FOLDER1)
    _ = mt.load_np('{}.npy'.format(path), ack=True)
    return


def save_load_npz_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.create_dir(TEMP_FOLDER1)
    path = '{}/{}.npz'.format(TEMP_FOLDER1, JUST_A_NAME)
    b = np.ones(shape=(2, 3, 29))
    c = np.ones(shape=(2, 3, 29))
    b_c = {'b': b, 'c': c}
    print(mt.to_str(b_c, '\tb_c'))
    mt.save_npz(b_c, path=path)
    b_c2 = mt.load_npz(path)
    print(mt.to_str(b_c2, '\tb_c2', rec=True))
    mt.delete_dir_with_files(TEMP_FOLDER1)
    return


def save_load_pkl_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.create_dir(TEMP_FOLDER1)
    path = '{}/{}.pkl'.format(TEMP_FOLDER1, JUST_A_NAME)
    a = {'2': 'a', 'b': 9, 'x': np.ones(shape=3)}
    print(mt.to_str(a, '\ta'))
    mt.save_pkl(data_dict=a, path=path)
    a2 = mt.load_pkl(path=path)
    print(mt.to_str(a2, '\ta2'))
    mt.delete_dir_with_files(TEMP_FOLDER1)
    return


def get_uniform_dist_by_dim_test():
    mt.get_function_name(ack=True, tabs=0)
    A = np.array([[1, 100], [7, 210], [3, 421]])
    lows, highs = mt.get_uniform_dist_by_dim(A)
    print(mt.to_str(A, '\tA'))
    print(mt.to_str(lows, '\tlows'))
    print(mt.to_str(highs, '\thighs'))
    A = A.tolist()
    print(mt.to_str(A, '\tA'))
    lows, highs = mt.get_uniform_dist_by_dim(A)
    print(mt.to_str(lows, '\tlows'))
    print(mt.to_str(highs, '\thighs'))
    A = mt.np_uniform(shape=(500, 2), lows=[3, 200], highs=[12, 681])
    print(mt.to_str(A, '\tA(lows=[3, 200],highs=[12, 681])'))
    lows, highs = mt.get_uniform_dist_by_dim(A)
    print(mt.to_str(lows, '\tlows'))
    print(mt.to_str(highs, '\thighs'))
    return


def get_normal_dist_by_dim_test():
    mt.get_function_name(ack=True, tabs=0)
    A = np.array([[1, 100], [7, 210], [3, 421]])
    means, stds = mt.get_normal_dist_by_dim(A)
    print(mt.to_str(A, '\tA'))
    print(mt.to_str(means, '\tmeans'))
    print(mt.to_str(stds, '\tstds'))
    A = A.tolist()
    print(mt.to_str(A, '\tA'))
    means, stds = mt.get_normal_dist_by_dim(A)
    print(mt.to_str(means, '\tmeans'))
    print(mt.to_str(stds, '\tstds'))
    A = mt.np_normal(shape=(500, 2), mius=[3, 200], stds=[12, 121])
    print(mt.to_str(A, '\tA(mius=[3, 200],stds=[12, 121])'))
    means, stds = mt.get_normal_dist_by_dim(A)
    print(mt.to_str(means, '\tmeans'))
    print(mt.to_str(stds, '\tstds'))
    return


def np_uniform_test():
    mt.get_function_name(ack=True, tabs=0)
    A = mt.np_uniform(shape=(500, 2), lows=[3, 200], highs=[12, 681])
    print(mt.to_str(A, '\tA(lows=[3, 200],highs=[12, 681])'))
    return


def np_normal_test():
    mt.get_function_name(ack=True, tabs=0)
    A = mt.np_normal(shape=(500, 2), mius=[3, 200], stds=[12, 121])
    print(mt.to_str(A, '\tA(mius=[3, 200],stds=[12, 121])'))
    return


def generate_new_data_from_old_test():
    mt.get_function_name(ack=True, tabs=0)
    print('\tgenerate uniform data example')
    old_data = mt.np_uniform(shape=(500, 2), lows=[3, 200], highs=[12, 681])
    print(mt.to_str(old_data, '\t\told_data(lows=[3, 200],highs=[12, 681])'))
    new_data = mt.generate_new_data_from_old(old_data, new_data_n=4000, dist='uniform')
    lows, highs = mt.get_uniform_dist_by_dim(new_data)
    print(mt.to_str(new_data, '\t\tnew_data'))
    print(mt.to_str(lows, '\t\tlows'))
    print(mt.to_str(highs, '\t\thighs'))

    print('\tgenerate normal data example')
    old_data = mt.np_normal(shape=(500, 2), mius=[3, 200], stds=[12, 121])
    print(mt.to_str(old_data, '\t\told_data(mius=[3, 200],stds=[12, 121])'))
    new_data = mt.generate_new_data_from_old(old_data, new_data_n=4000, dist='normal')
    means, stds = mt.get_normal_dist_by_dim(new_data)
    print(mt.to_str(new_data, '\t\tnew_data'))
    print(mt.to_str(means, '\t\tmeans'))
    print(mt.to_str(stds, '\t\tstds'))
    return


def np_random_integers_test():
    mt.get_function_name(ack=True, tabs=0)
    random_ints = mt.np_random_integers(low=5, high=20, size=(2, 3))
    print(mt.to_str(random_ints, '\trandom_ints from 5-20'))
    return


def augment_x_y_numpy_test():
    mt.get_function_name(ack=True, tabs=0)
    X = mt.np_random_integers(low=5, high=20, size=(10, 3))
    Y = mt.np_random_integers(low=0, high=10, size=(10,))
    print(mt.to_str(X, '\tX'))
    print(mt.to_str(Y, '\tY'))
    A = mt.augment_x_y_numpy(X, Y)
    print(mt.to_str(A, '\tA'))
    return


def de_augment_numpy_test():
    mt.get_function_name(ack=True, tabs=0)
    A = mt.np_random_integers(low=5, high=20, size=(10, 4))
    print(mt.to_str(A, '\tA'))
    X, Y = mt.de_augment_numpy(A)
    print(mt.to_str(X, '\tX'))
    print(mt.to_str(Y, '\tY'))
    return


def nCk_test():
    mt.get_function_name(ack=True, tabs=0)
    A = np.random.randint(low=-10, high=10, size=(3, 2))
    print(mt.to_str(A, '\tA'))

    # let's iterate on every 2 different indices of A
    combs_count = mt.nCk(len(A), k=2, as_int=True)
    print('\t{}C2={}:'.format(len(A), combs_count))  # result is 3

    combs_list = mt.nCk(len(A), k=2)  # result is [[0, 1], [0, 2], [1, 2]]
    for i, comb in enumerate(combs_list):
        print('\t\tcomb {}={}. A[comb]={}'.format(i, comb, A[comb].tolist()))
    return


def redirect_std_test():
    mt.get_function_name(ack=True, tabs=0)
    old_stdout, summary_str = mt.redirect_std_start()
    print('\t\tbla bla bla')
    print('\t\tline2')
    string = mt.redirect_std_finish(old_stdout, summary_str)
    print('\tcaptured output:')
    print(string, end='')  # there is '\n' at the end of the last line
    return


def get_line_number_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_line_number(ack=True)
    return


def get_function_name_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_function_name(ack=True)
    return


def get_file_name_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_file_name(ack=True)
    return


def get_base_file_name_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_base_file_name(ack=True)
    return


def get_function_name_and_line_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_function_name_and_line(ack=True)
    return


def get_base_file_and_function_name_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.get_base_file_and_function_name(ack=True)
    return


def add_color_test():
    mt.get_function_name(ack=True, tabs=0)
    print(mt.add_color(string='\tred ,bold and underlined', ops=['Red', 'bold', 'underlined']))
    print('\t{}'.format(mt.add_color(string='blue ,bold and underlined', ops=['BlUe', 'bo', 'un'])))
    print(mt.add_color(string='\tjust bold', ops='bold'))
    print(mt.add_color(string='\treverse color and bg color', ops='re'))
    print(mt.add_color(string='\tred with background_dark_gray', ops=['red', 'background_dark_gray']))
    print('\t{}'.format(mt.add_color(string='background_light_yellow', ops='background_light_yellow')))
    print(mt.add_color(string='\tblack and background_magenta', ops=['black', 'background_magenta']))
    my_str = 'using mt.to_str()'
    my_str = mt.to_str(var=my_str, title='\t{}'.format(my_str))
    print(mt.add_color(my_str, ops='Green'))
    return


def logger_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.create_dir(TEMP_FOLDER1)
    path = '{}/{}_log_{}.txt'.format(TEMP_FOLDER1, JUST_A_NAME, mt.get_time_stamp(format_s='%Y_%m_%d_%H_%M_%S'))
    mt.init_logger(logger_path=path)
    mt.log_print(line='\tline 1')
    mt.flush_logger()
    mt.log_print(line='line 2', tabs=1)
    mt.log_print(line='line 3', tabs=3)
    mt.close_logger()

    print('\treading from {}'.format(path))
    file1 = open(path, 'r')
    Lines = file1.readlines()
    file1.close()
    for i, line in enumerate(Lines):
        print("\t\tLine{}: {}".format(i + 1, line.rstrip()))
    mt.delete_dir_with_files(TEMP_FOLDER1)
    return


def create_and_delete_empty_dir_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.create_dir(dir_path=TEMP_FOLDER1)
    mt.delete_empty_dir(dir_path=TEMP_FOLDER1)

    mt.create_dir(dir_path='{}/{}'.format(TEMP_FOLDER1, os.path.basename(TEMP_FOLDER1)))

    mt.delete_empty_dir(dir_path=TEMP_FOLDER1)  # will fail
    # remove empty dir 1 by 1
    mt.delete_empty_dir(dir_path='{}/{}'.format(TEMP_FOLDER1, os.path.basename(TEMP_FOLDER1)))  # will work
    mt.delete_empty_dir(dir_path=TEMP_FOLDER1)  # will work
    return


def create_and_delete_dir_with_files_test():
    mt.get_function_name(ack=True, tabs=0)
    base_path = TEMP_FOLDER1
    inner_dir = '{}/{}'.format(base_path, os.path.basename(TEMP_FOLDER1))
    create_demo_file(path='{}/{}'.format(base_path, DEMO_FILE))
    create_demo_file(path='{}/v2_{}'.format(base_path, DEMO_FILE))
    create_demo_file(path='{}/{}'.format(inner_dir, DEMO_FILE))
    create_demo_file(path='{}/v2_{}'.format(inner_dir, DEMO_FILE))

    # mt.delete_empty_dir(dir_path=base_path)  # will fail
    mt.delete_dir_with_files(dir_path=base_path)
    return


def find_files_in_folder_test():
    test_f = TEMP_FOLDER1
    txt1 = '{}/{}'.format(test_f, DEMO_FILE)
    txt2 = '{}/v2_{}'.format(test_f, DEMO_FILE)
    zip1 = '{}/{}.zip'.format(test_f, JUST_A_NAME)
    create_demo_file(txt1)
    create_demo_file(txt2)
    create_demo_file(zip1)
    mt.find_files_in_folder(dir_path=test_f, file_suffix='.txt', ack=True)
    mt.find_files_in_folder(dir_path=test_f, file_suffix='.zip', ack=True)
    mt.delete_dir_with_files(test_f)
    return


def move_file_test():
    mt.get_function_name(ack=True, tabs=0)
    file_src = '{}/{}'.format(TEMP_FOLDER1, DEMO_FILE)
    file_dst = '{}/v2_{}'.format(TEMP_FOLDER1, DEMO_FILE)
    create_demo_file(file_src)
    mt.move_file(file_src=file_src, file_dst=file_dst)
    mt.delete_dir_with_files(TEMP_FOLDER1)
    return


def copy_file_test():
    mt.get_function_name(ack=True, tabs=0)
    file_src = '{}/{}'.format(TEMP_FOLDER1, DEMO_FILE)
    file_dst = '{}/v2_{}'.format(TEMP_FOLDER1, DEMO_FILE)
    create_demo_file(file_src)
    mt.copy_file(file_src=file_src, file_dst=file_dst)
    mt.delete_dir_with_files(TEMP_FOLDER1)
    return


def delete_file_test():
    mt.get_function_name(ack=True, tabs=0)
    path = '{}/{}'.format(TEMP_FOLDER1, DEMO_FILE)
    create_demo_file(path)
    mt.delete_dir_with_files(TEMP_FOLDER1)
    return


def delete_files_test():
    mt.get_function_name(ack=True, tabs=0)
    path1 = '{}/v1_{}'.format(TEMP_FOLDER1, DEMO_FILE)
    path2 = '{}/v2_{}'.format(TEMP_FOLDER1, DEMO_FILE)
    create_demo_file(path1)
    create_demo_file(path2)
    mt.delete_files(files=[path1, path2])
    mt.delete_dir_with_files(TEMP_FOLDER1)
    return


def sleep_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.sleep(seconds=2, ack=True, tabs=1)
    return


def reverse_tuple_or_list_test():
    mt.get_function_name(ack=True, tabs=0)
    my_tuple = (0, 0, 255)
    print(mt.to_str(my_tuple, '\tmy_tuple'))
    print(mt.to_str(mt.reverse_tuple_or_list(my_tuple), '\tmy_tuple_reversed'))
    my_list = [0, 0, 255]
    print(mt.to_str(my_list, '\tmy_list'))
    print(mt.to_str(mt.reverse_tuple_or_list(my_list), '\tmy_list_reversed'))
    return


def round_list_test():
    mt.get_function_name(ack=True, tabs=0)
    li = [1.23123123, 12.123123123123123123, 1.2, 1.0, 6e-06]
    print(mt.to_str(var=li, title='\torigin', fp=-1))
    print(mt.to_str(var=mt.round_list(li, fp=1), title='\tfp=1', fp=-1))
    print(mt.to_str(var=mt.round_list(li, fp=6), title='\tfp=6', fp=-1))

    li = [[1.23123123, 12.123123123123123123, 1.2, 1.0, 6e-06, 2]]
    print(mt.to_str(var=li, title='\torigin', fp=-1))
    print(mt.to_str(var=mt.round_list(li, fp=6), title='\tfp=6', fp=-1))

    l2 = [5e-05, 0.00014, 5e-10, 0.0001, 6e-07, 5e-05, 8e-05, 6e-05, 5e-05, 5e-05, 2]
    print(mt.to_str(var=l2, title='\torigin', fp=-1))
    print(mt.to_str(var=mt.round_list(l2, fp=5), title='\tfp=5', fp=-1))

    l2 = ['x', 'y', 'z', 0.3]
    print(mt.to_str(var=l2, title='\torigin', fp=-1))
    print(mt.to_str(var=mt.round_list(l2, fp=5, warn=True), title='\tfp=5', fp=-1))
    return


def round_tuple_test():
    mt.get_function_name(ack=True, tabs=0)
    li = (1.23123123, 12.123123123123123123, 1.2, 1.0)
    print(mt.to_str(var=li, title='\torigin', fp=-1))
    print(mt.to_str(var=mt.round_tuple(li, fp=1), title='\tfp=1', fp=-1))
    print(mt.to_str(var=mt.round_tuple(li, fp=3), title='\tfp=3', fp=-1))
    l2 = (5e-05, 0.00014, 5e-10, 0.0001, 6e-07, 5e-05, 8e-05, 6e-05, 5e-05, 5e-05)
    print(mt.to_str(var=l2, title='\torigin', fp=-1))
    print(mt.to_str(var=mt.round_tuple(l2, fp=5), title='\tfp=5', fp=-1))
    l2 = ('x', 'y', 'z')
    print(mt.to_str(var=l2, title='\torigin', fp=-1))
    print(mt.to_str(var=mt.round_tuple(l2, fp=5, warn=True), title='\tfp=5', fp=-1))
    return


def shuffle_np_array_test():
    mt.get_function_name(ack=True, tabs=0)
    A = mt.np_normal(shape=(6,), mius=3.5, stds=10.2)
    print(mt.to_str(A, '\tA'))
    A = mt.shuffle_np_array(A)
    print(mt.to_str(A, '\tA'))
    return


def shuffle_np_arrays_test():
    mt.get_function_name(ack=True, tabs=0)
    A = np.array([1, 2, 3, 4, 5, 6])
    B = np.array([[1, 1], [2, 2], [3, 3], [4, 4], [5, 5], [6, 6]])
    print(mt.to_str(A, '\tA'))
    print(mt.to_str(B, '\tB'))
    A, B = mt.shuffle_np_arrays(
        arr_tuple=(A, B)
    )
    print(mt.to_str(A, '\tA'))
    print(mt.to_str(B, '\tB'))
    return


def array_info_print_test():
    B = np.array([[1, 1], [2, 2], [3, 3], [4, 4], [5, 5], [6, 6]])
    mt.array_info_print(B, 'B')
    return


def get_key_by_value_test():
    mt.get_function_name(ack=True, tabs=0)
    j = {"x": 3, "a": "dx"}
    print('\tfirst key that has value 3 is {}'.format(mt.get_key_by_value(j, value=3)))
    print('\tfirst key that has value "dx" is {}'.format(mt.get_key_by_value(j, value="dx")))
    return


def to_hex_and_bin_test():
    mt.get_function_name(ack=True, tabs=0)
    variable = 'no meaning to to content'
    print('\taddress of variable is 0d{}'.format(id(variable)))
    print('\taddress of variable is 0x{}'.format(mt.to_hex(id(variable))))
    print('\taddress of variable is 0b{}'.format(mt.to_bin(id(variable))))
    return


def dict_as_table_test():
    mt.get_function_name(ack=True, tabs=0)
    table = {'gilad': 3, 'a': 11233.1213, 'aasdasd': 9913123, 'b': 'hello'}
    mt.dict_as_table(table=table, title='my table', fp=2, tabs=1)
    # mt.dict_as_table(table=table, title='my table', fp=5, tabs=2)
    return


def is_same_type_test():
    mt.get_function_name(ack=True, tabs=0)
    li = ['s', 123, 'a', 'b']
    print('\tis li {} homogeneous ? {}'.format(li, mt.is_same_type(li)))
    li = ['s', 'a', 'b']
    print('\tis li {} homogeneous ? {}'.format(li, mt.is_same_type(li)))
    t = (1.2, 123, 12, 13)
    print('\tis t {} homogeneous ? {}'.format(t, mt.is_same_type(t)))
    t = (1, 123, 12, 13)
    print('\tis t {} homogeneous ? {}'.format(t, mt.is_same_type(t)))
    return


def hard_disc_test():
    mt.get_function_name(ack=True, tabs=0)
    print(mt.hard_disc(one_liner=True, tabs=1))
    print(mt.hard_disc(one_liner=False, tabs=1))
    return


def ram_size_test():
    mt.get_function_name(ack=True, tabs=0)
    print(mt.ram_size(one_liner=True, tabs=1))
    print(mt.ram_size(one_liner=False, tabs=1))
    return


def cpu_info_test():
    mt.get_function_name(ack=True, tabs=0)
    print(mt.cpu_info(one_liner=True, tabs=1))
    print(mt.cpu_info(one_liner=False, tabs=1))
    return


def file_or_folder_size_test():
    mt.get_function_name(ack=True, tabs=0)
    mt.file_or_folder_size('{}.txt'.format(FAKE_FILE), ack=True, tabs=1)

    path = '{}/{}'.format(TEMP_FOLDER1, DEMO_FILE)
    create_demo_file(path)
    mt.file_or_folder_size(path, ack=True, tabs=1)
    mt.delete_file(path)

    create_demo_file('{}/{}'.format(TEMP_FOLDER1, DEMO_FILE))
    create_demo_file('{}/v2_{}'.format(TEMP_FOLDER1, DEMO_FILE))
    create_demo_file('{}/{}/{}'.format(TEMP_FOLDER1, os.path.basename(TEMP_FOLDER1), DEMO_FILE))
    mt.file_or_folder_size(TEMP_FOLDER1, ack=True, tabs=1)
    mt.delete_dir_with_files(TEMP_FOLDER1)
    return


def compress_and_extract_test():
    mt.get_function_name(ack=True, tabs=0)
    # single file
    print('\tSingle file tests:')
    f_name = JUST_A_NAME
    file_to_compress_path = '{}/{}.txt'.format(TEMP_FOLDER1, f_name)
    create_demo_file(path=file_to_compress_path)  # temp file

    for file_t in ['zip', '7z', 'tar']:
        print('\t{}:'.format(file_t))
        compressed_file_path = '{}/{}'.format(TEMP_FOLDER1, f_name)
        mt.compress_file_or_folder(src=file_to_compress_path, dst_path=compressed_file_path, file_type=file_t, ack=True,
                                   tabs=2)
        compressed_file_path += '.{}'.format(file_t)
        dst_folder = '{}/{}_{}'.format(TEMP_FOLDER1, f_name, file_t)
        mt.extract_file(src=compressed_file_path, dst_folder=dst_folder, file_type=file_t, tabs=2)
        mt.delete_file(compressed_file_path, tabs=2)  # remove compressed
        mt.delete_dir_with_files(dst_folder, tabs=2)  # remove extracted

    mt.delete_file(file_to_compress_path)  # remove temp file

    # folder
    print('\tFolder tests:')
    folder_to_compress_path = '{}/{}'.format(TEMP_FOLDER1, os.path.basename(TEMP_FOLDER1))
    f1 = '{}/{}'.format(folder_to_compress_path, DEMO_FILE)
    create_demo_file(path=f1)  # temp file 1
    f2 = '{}/v2_{}.txt'.format(folder_to_compress_path, DEMO_FILE)
    create_demo_file(path=f2)  # temp file
    for file_t in ['zip', '7z', 'tar']:
        print('\t{}:'.format(file_t))
        compressed_folder_path = '{}/{}'.format(TEMP_FOLDER1, os.path.basename(TEMP_FOLDER1))
        mt.compress_file_or_folder(src=folder_to_compress_path, dst_path=compressed_folder_path, file_type=file_t,
                                   ack=True, tabs=2)
        compressed_folder_path += '.{}'.format(file_t)
        dst_folder = '{}/{}_{}'.format(TEMP_FOLDER1, os.path.basename(TEMP_FOLDER1), file_t)
        mt.extract_file(src=compressed_folder_path, dst_folder=dst_folder, file_type=file_t, tabs=2)
        mt.delete_file(compressed_folder_path, tabs=2)  # remove compressed
        mt.delete_dir_with_files(dst_folder, tabs=2)  # remove extracted
    mt.delete_dir_with_files(TEMP_FOLDER1)  # remove temp folder with temp file 1 and 2
    return


def classFPS_test():
    mt.get_function_name_and_line(ack=True, tabs=0)
    fps = mt.FPS(summary_title='classFPS_test')
    tabs = 1
    for t in range(3):
        fps.start(ack_progress=True, tabs=tabs)
        # do_work of round t
        mt.sleep(seconds=0.03)
        fps.update(ack_progress=True, tabs=tabs + 1)
    fps.finalize(tabs)
    return


def classFPS_many_test():
    mt.get_function_name_and_line(ack=True, tabs=0)
    # if you measure more than 1
    fps_list = [mt.FPS(summary_title='work1'), mt.FPS(summary_title='work2')]
    tabs = 1
    for t in range(3):
        for i in range(len(fps_list)):
            fps_list[i].start(ack_progress=True, tabs=tabs, with_title=True)
            # do_work of round t
            mt.sleep(seconds=0.03)
            fps_list[i].update(ack_progress=True, tabs=tabs + 1, with_title=True)
    for fps in fps_list:
        fps.finalize(tabs)
    return


def try_say_welcome():
    mt.get_function_name_and_line(ack=True, tabs=0)
    try:
        from wizzi_utils.tts.tts import MachineBuddy
        MachineBuddy.speak('Welcome to wizzi utils package', block=True)  # Static use:
        # MachineBuddy.speak('There is a dog pooping outside')  # Static use:
    except(ImportError, ModuleNotFoundError) as e:
        mt.exception_error('Can\'t do try_say_welcomec. {}'.format(e), real_exception=True)
    return


def get_linkable_exception_test():
    mt.get_function_name_and_line(ack=True, tabs=0)
    try:
        1 / 0
    except Exception as e:
        mt.exception_error(e, real_exception=True, tabs=1)
    return


def test_all():
    print('{}{}:'.format('-' * 5, mt.get_base_file_and_function_name(depth=1)))
    try_say_welcome()
    timer_test()
    timer_action_test()
    get_time_stamp_test()
    get_pc_name_test()
    get_mac_address_test()
    get_cuda_version_test()
    get_env_variables_test()
    set_env_variable_test()
    get_env_variable_test()
    del_env_variable_test()
    make_cuda_invisible_test()
    profiler_test()
    os_test()
    get_wizzi_utils_root_test()
    main_wrapper_test()
    to_str_test()
    save_load_np_test()
    save_load_npz_test()
    save_load_pkl_test()
    get_uniform_dist_by_dim_test()
    get_normal_dist_by_dim_test()
    np_uniform_test()
    np_normal_test()
    generate_new_data_from_old_test()
    np_random_integers_test()
    augment_x_y_numpy_test()
    de_augment_numpy_test()
    nCk_test()
    redirect_std_test()
    get_line_number_test()
    get_function_name_test()
    get_file_name_test()
    get_base_file_name_test()
    get_function_name_and_line_test()
    get_base_file_and_function_name_test()
    add_color_test()
    logger_test()
    create_and_delete_empty_dir_test()
    create_and_delete_dir_with_files_test()
    move_file_test()
    copy_file_test()
    find_files_in_folder_test()
    delete_file_test()
    delete_files_test()
    sleep_test()
    reverse_tuple_or_list_test()
    round_list_test()
    round_tuple_test()
    shuffle_np_array_test()
    shuffle_np_arrays_test()
    array_info_print_test()
    get_key_by_value_test()
    to_hex_and_bin_test()
    dict_as_table_test()
    is_same_type_test()
    hard_disc_test()
    ram_size_test()
    cpu_info_test()
    compress_and_extract_test()
    file_or_folder_size_test()
    classFPS_test()
    get_linkable_exception_test()
    print('{}'.format('-' * 20))
    return



try:
    from wizzi_utils.models.cv2_models import object_detection as od
except (ModuleNotFoundError, AttributeError, ImportError) as e:
    pass

try:
    from wizzi_utils.models.cv2_models import pose_detection as pd
except (ModuleNotFoundError, AttributeError, ImportError) as e:
    pass

try:
    from wizzi_utils.models.cv2_models import tracking as tr
except (ModuleNotFoundError, AttributeError, ImportError) as e:
    pass



from wizzi_utils.misc import misc_tools as mt
from wizzi_utils.misc.test import test_misc_tools as mtt
from wizzi_utils.open_cv import open_cv_tools as cvt
from wizzi_utils.open_cv.test import test_open_cv_tools as cvtt
from wizzi_utils.pyplot import pyplot_tools as pyplt
import numpy as np
# noinspection PyPackageRequirements
import cv2
from wizzi_utils.models.cv2_models import tracking as tr
from wizzi_utils.models.test.shared_code_for_tests import TR_MODEL, TR_VID_TEST, DEBUG_DETECTIONS


def tr_run(
        model: tr.Cv2Tracker,
        cv_img: np.array,
        frame_idx: int,
        fps: mt.FPS
):
    fps.start()
    detections = model.track_frame(cv_img, ack=DEBUG_DETECTIONS, tabs=0, frame_idx=frame_idx)
    fps.update(ack_progress=True, tabs=1, with_title=True)

    model.draw_detections(
        detections,
        cv_img=cv_img,
        colors_d={
            'default_bbox_color': 'lime',
            'bbox_color_1': 'red',  # custom bbox color for tracker id == 1
        },
        tracks_dict={
            'default_tracks_color': 'blue',
            'len_tracks': 100,
            'radius': 1,
            'thickness': -1,
            'tracks_color_1': 'yellow'  # custom tracks color for tracker id == 1
        },
        # header_tl={'text': 'wizzi_utils', 'x_offset': 0, 'text_color': 'aqua', 'with_rect': True, 'bg_color': 'black',
        #            'bg_font_scale': 1},
        header_bl={'text': '{}'.format(model.tracker_type), 'x_offset': 0, 'text_color': 'white',
                   'with_rect': True, 'bg_color': 'black', 'bg_font_scale': 1},
        header_tr={'text': '{:.2f} FPS'.format(fps.get_fps()), 'x_offset': 190, 'text_color': 'r', 'with_rect': False,
                   'bg_color': None, 'bg_font_scale': 1},
        header_br={'text': mt.get_time_stamp(format_s='%Y_%m_%d'), 'x_offset': 200, 'text_color': 'aqua',
                   'with_rect': True, 'bg_color': 'black', 'bg_font_scale': 1},
    )

    return


def tr_Model_video_test(
        model_name: str,
        vid_path: str,
        work_every_x_frames: int = 1,
        in_dims_w: int = None
):
    mt.get_function_name(ack=True, tabs=0)

    cam = cv2.VideoCapture(vid_path)
    if cam is not None:
        img_w, img_h = cvt.get_dims_from_cap(cam)
        print('\tFrame dims {}x{}'.format(img_w, img_h))
        if in_dims_w is None:
            in_dims = (img_w, img_h)  # work on orig image
        else:
            # work on w=in_dims_w and h derived from w
            in_dims = cvt.get_aspect_ratio_h(img_w=img_w, img_h=img_h, new_w=in_dims_w)

        model = tr.Cv2Tracker(model_obj=tr.Trackers[model_name])
        print(model.to_string())

        fps = mt.FPS(summary_title='{}'.format(model.tracker_type))
        max_frames = cvt.get_frames_from_cap(cam)
        title = 'cv tracker'
        for i in range(max_frames):
            success, cv_img = cam.read()
            if i % work_every_x_frames != 0:  # s
                # do only x frames
                continue
            if success:
                cv_img = cv2.resize(cv_img, in_dims)
                tr_run(model, cv_img, frame_idx=i, fps=fps)
                if i == 0:  # first iter take much longer - measure from second iter
                    fps.clear()
                k = cvt.display_open_cv_image(
                    cv_img,
                    ms=1,
                    title=title,
                    loc=pyplt.Location.CENTER_CENTER.value,
                    resize=None,
                    # resize=(1280, 720),
                    header='{}/{}'.format(i + 1, max_frames),
                    save_path=None
                )
                if k == ord('q'):
                    mt.exception_error('q was clicked. breaking loop')
                    break
                if k == ord('s'):
                    if isinstance(model, tr.Cv2Tracker):
                        model.add_obj_bbox_roi(title, cv_img, ack=DEBUG_DETECTIONS, frame_idx=i)

                if vid_path.endswith('woman_yoga.mp4'):
                    # ADD manually object that i know were there to save you dealing with ROI
                    if i == 14:
                        xywh = (175, 66, 194, 219)  # woman in original image space
                        bbox = model.change_dims(xywh=xywh, old_dims=(img_w, img_h), new_dims=in_dims)
                        bbox = model.tuple_to_int(bbox)
                        model.add_obj_bbox_manual(cv_img, xywh=bbox, frame_idx=i, ack=DEBUG_DETECTIONS)
                    # if i == 40:  # checking if active == False works fine
                    #     model.get_tracker_dict(tracker_id=0)['active'] = False
                    if i == 36:
                        xywh = (241, 285, 68, 70)  # cat in original image space
                        bbox = model.change_dims(xywh=xywh, old_dims=(img_w, img_h), new_dims=in_dims)
                        bbox = model.tuple_to_int(bbox)
                        model.add_obj_bbox_manual(cv_img, xywh=bbox, frame_idx=i, ack=DEBUG_DETECTIONS)
            else:
                mt.exception_error(e='failed to grab frame {}'.format(i))
                continue
        fps.finalize()
        cv2.destroyAllWindows()
    return


def test_all():
    print('{}{}:'.format('-' * 5, mt.get_base_file_and_function_name()))
    if TR_VID_TEST:
        vid_path = cvtt.get_vid_from_web(name=mtt.WOMAN_YOGA)
        tr_Model_video_test(
            model_name=TR_MODEL,  # slower but better
            vid_path=vid_path,
            work_every_x_frames=1,
            # in_dims_w=500,  # if None - work on original image size
        )
    print('{}'.format('-' * 20))
    return



try:
    from wizzi_utils.models import models_configs
    from wizzi_utils.models import base_models
    from wizzi_utils.models import cv2_models
    from wizzi_utils.models import tflite_models
except (ModuleNotFoundError, AttributeError, ImportError) as e:
    pass

from wizzi_utils.models import test



from distutils.core import setup
# noinspection PyUnresolvedReferences
import setuptools

"""
see https://docs.python.org/3/distutils/setupscript.html
"""

# Read in README.md for our long_description
import os

cwd = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(cwd, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='wizzi_utils',
    packages=[  # main package and sub packages
        'wizzi_utils',  # package name
        'wizzi_utils/misc',  # main sub package
        'wizzi_utils/misc/test',
        # 'wizzi_utils/algorithms',
        # 'wizzi_utils/algorithms/test',
        # 'wizzi_utils/coreset',
        # 'wizzi_utils/coreset/test',
        'wizzi_utils/google',
        'wizzi_utils/google/test',
        'wizzi_utils/json',
        'wizzi_utils/json/test',
        'wizzi_utils/open_cv',
        'wizzi_utils/open_cv/test',
        'wizzi_utils/pyplot',
        'wizzi_utils/pyplot/test',
        'wizzi_utils/socket',
        'wizzi_utils/socket/test',
        'wizzi_utils/torch',
        'wizzi_utils/torch/test',
        # 'wizzi_utils/tensorflow',
        # 'wizzi_utils/tensorflow/test',
        'wizzi_utils/tflite',
        'wizzi_utils/tflite/test',
        'wizzi_utils/tts',
        'wizzi_utils/tts/test',
        'wizzi_utils/models',
        'wizzi_utils/models/cv2_models',
        'wizzi_utils/models/tflite_models',
        'wizzi_utils/models/test',
    ],
    version='6.8.0',
    license='MIT',  # https://help.github.com/articles/licensing-a-repository
    description='Handy Tools for Developers',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Gilad Eini',
    author_email='giladEini@gmail.com',
    url='https://github.com/2easy4wizzi/2021wizzi_utils',  # link to github
    download_url='https://github.com/2easy4wizzi/2021wizzi_utils/archive/refs/tags/v_6.8.0.tar.gz',
    keywords=[  # Keywords that define your package best
        'misc tools',
        'common algorithms',
        'coreset tools',
        'json tools',
        'open cv tools',
        'pyplot tools',
        'socket tools',
        'torch tools',
    ],
    install_requires=[
        'datetime',
        'typing',
        'numpy',
        'psutil',
        'matplotlib',
    ],
    classifiers=[
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3.6'
    ],
    platforms='windows',
)



from wizzi_utils.misc import misc_tools as mt
from wizzi_utils.misc.test import test_misc_tools as mtt
from wizzi_utils.open_cv.test import test_open_cv_tools as cvtt
from wizzi_utils.models.models_configs import MODELS_CONFIG
from wizzi_utils.models.models_configs import ModelType
from wizzi_utils.models.tflite_models import pose_detection as pd
from wizzi_utils.models.test.shared_code_for_tests import CAM_FRAMES, IMAGE_MS, VID_X_FRAMES_TFL
from wizzi_utils.models.test.shared_code_for_tests import PD_OP, PD_IM_TEST, PD_CAM_TEST, PD_VID_TEST
from wizzi_utils.models.test.shared_code_for_tests import od_or_pd_Model_image_test
from wizzi_utils.models.test.shared_code_for_tests import od_or_pd_Model_video_test
from wizzi_utils.models.test.shared_code_for_tests import od_or_pd_Model_cam_test


def __get_pd_dict(op: int = 1):
    if op == 1:
        pd_solo = {
            'model_names': ['pose_landmark_lite'],
            # 'model_names': ['pose_landmark_full'],
            # 'model_names': ['pose_landmark_heavy'],
            # 'model_names': ['posenet'],
            'dis_size': (640, 480),
            'grid': (1, 1),
        }
        pd_meta_dict = pd_solo
    elif op == 2:
        pd_dual = {
            'model_names': [
                'pose_landmark_lite',
                # 'pose_landmark_full',
                # 'pose_landmark_heavy',
                'posenet',
            ],
            'dis_size': (640 * 2, 480),
            'grid': (1, 2),
        }
        pd_meta_dict = pd_dual
    elif op == 3:
        all_models_names = pd.PdBaseModel.get_pose_detection_models(m_type='Tfl', ack=False)
        pd_all_models = {
            'model_names': all_models_names,
            'dis_size': 'fs',
            'grid': (2, 2),
        }
        pd_meta_dict = pd_all_models
    else:
        pd_meta_dict = None
    return pd_meta_dict


def _get_models(model_names: list) -> list:
    models = []
    for model_name in model_names:
        model_cfg = MODELS_CONFIG[model_name]
        m_save_dir = '{}/{}/{}'.format(mtt.MODELS, MODELS_CONFIG[model_name]['job'], model_name)
        if model_cfg['model_type'] == ModelType.PdTflNormal.value:
            model = pd.TflPdModel(
                save_load_dir=m_save_dir,
                model_name=model_name,
                # allowed_joint_names=['nose', 'leftEyeInside', 'leftEye']
            )
        elif model_cfg['model_type'] == ModelType.PdTflPoseNet.value:
            model = pd.TflPdModelPoseNet(
                save_load_dir=m_save_dir,
                model_name=model_name,
            )
        else:
            model = None
            mt.exception_error('model type not found')
            exit(-1)
        print(model.to_string(tabs=1))
        models.append(model)
    return models


def test_all():
    print('{}{}:'.format('-' * 5, mt.get_base_file_and_function_name()))
    pd_meta_dict = __get_pd_dict(op=PD_OP)
    models = _get_models(pd_meta_dict['model_names'])

    # POSE DETECTION TESTS
    if PD_IM_TEST:
        # cv_img = cvtt.load_img_from_web(mtt.FACES, ack=False)
        # cv_img = cvtt.load_img_from_web(mtt.HAND, ack=False)  # if testing 'hand_pose', use HAND image
        cv_img = cvtt.load_img_from_web(mtt.F_MODEL, ack=False)
        # cv_img = cvt.load_img(path='{}/Input2/77.jpg'.format(mtt.IMAGES_PATH))
        # cv_img = cvt.load_img(path='{}/Input2/cam1_0.jpg'.format(mtt.IMAGES_PATH))
        # cv_img = cvt.load_img(path='{}/Input2/90.jpg'.format(mtt.IMAGES_PATH))
        od_or_pd_Model_image_test(
            cv_img=cv_img,
            models=models,
            ms=IMAGE_MS,
            dis_size=pd_meta_dict['dis_size'],
            grid=pd_meta_dict['grid'],
        )
    if PD_CAM_TEST:
        od_or_pd_Model_cam_test(
            models=models,
            max_frames=CAM_FRAMES,
            dis_size=pd_meta_dict['dis_size'],
            grid=pd_meta_dict['grid'],
        )
    if PD_VID_TEST:
        vid_path = cvtt.get_vid_from_web(name=mtt.WOMAN_YOGA)
        od_or_pd_Model_video_test(
            models=models,
            vid_path=vid_path,
            work_every_x_frames=VID_X_FRAMES_TFL,
            dis_size=pd_meta_dict['dis_size'],
            grid=pd_meta_dict['grid'],
        )
    print('{}'.format('-' * 20))
    return



import numpy as np
import math
from tflite_runtime.interpreter import Interpreter
from wizzi_utils.misc import misc_tools as mt
from wizzi_utils.open_cv import open_cv_tools as cvt
from wizzi_utils.models import models_configs as cfg
from wizzi_utils.models.base_models import OdBaseModel
# noinspection PyPackageRequirements
import cv2


class TflOdModel(OdBaseModel):
    def __init__(
            self,
            save_load_dir: str,
            model_name: str,
            allowed_class: list = None,
            threshold: float = None,
            nms: dict = None,
            check_type: bool = True
    ):
        """
        :param save_load_dir: where the model is saved (or will be if not exists)
        :param model_name: valid name in MODEL_CONF.keys()
        :param threshold: only detection above this threshold will be pass first filter
        :param nms: non maximum suppression threshold 2 thresholds
            score_threshold and nms_threshold
            can be None in the cfg - if None pass and None on cfg: no nms
        :param allowed_class: ignore rest of class. list of strings
        example:
        model = od.TflOdModel(
            save_load_dir=m_save_dir,
            model_name=model_name,
        )
        """
        super().__init__(save_load_dir=save_load_dir, model_name=model_name, allowed_class=allowed_class)
        if check_type and not self.model_type_valid(self.model_cfg['model_type'], cfg.ModelType.OdTflNormal.value):
            exit(-1)
        if threshold is not None:
            self.model_cfg['threshold'] = threshold
        if nms is not None:
            self.model_cfg['nms'] = nms

        model_fp = "{}/{}.tflite".format(self.local_path, self.model_name)
        self._download_if_needed(local_path=model_fp, url_dict=self.model_cfg['tflite'])
        self.model_size = mt.file_or_folder_size(model_fp)
        # print('Loading {}(size {}, {} classes)'.format(self.model_name, self.model_size, len(self.labels)))
        self.interpreter = Interpreter(model_path=model_fp, num_threads=4)
        # allocate input output placeholders
        self.interpreter.allocate_tensors()
        self.input_details = self.interpreter.get_input_details()
        self.output_details = self.interpreter.get_output_details()
        _, input_height, input_width, _ = self.interpreter.get_input_details()[0]['shape']
        self.model_cfg['in_dims'] = (input_width, input_height)
        self.model_cfg['input_type'] = self.interpreter.get_input_details()[0]['dtype']
        quantization = self.interpreter.get_input_details()[0]['quantization']
        # need to normalize if model doesn't do quantization (then quantization == (0.0, 0))
        # else e.g. quantization == (0.0078125, 128) - no need to normalize
        self.model_cfg['normalize_RGB'] = True if quantization == (0.0, 0) else False

        # # you can print this to get more details on the model
        # mt.dict_as_table(self.interpreter.get_input_details()[0], title='input')
        # mt.dict_as_table(self.interpreter.get_output_details()[0], title='output')
        # mt.dict_as_table(self.interpreter.get_tensor_details()[0], title='tensor')
        return

    def to_string(self, tabs: int = 1) -> str:
        tabs_s = tabs * '\t'
        string = '{}{}'.format(tabs_s, mt.add_color(string='TflOdModel:', ops='underlined'))
        string += '\n\t{}name= {} (size {})'.format(tabs_s, self.model_name, self.model_size)
        string += '\n\t{}local_path= {}'.format(tabs_s, self.local_path)
        string += '\n\t{}{}'.format(tabs_s, mt.to_str(self.allowed_classes, 'allowed_classes'))
        string += '\n{}'.format(mt.dict_as_table(self.model_cfg, title='cfg', fp=6, ack=False, tabs=tabs + 1))
        return string

    def prepare_input(self, cv_img: np.array) -> np.array:
        """
        :param cv_img:
        resize and change dtype to predefined params
        :return:
        """
        img_RGB = cvt.BGR_img_to_RGB(cv_img)
        if self.model_cfg['normalize_RGB']:
            # normalization is done via the authors of the MobileNet SSD implementation
            center = 127.5
            img_RGB = (img_RGB - center) / center  # normalize image
        img = cv2.resize(img_RGB, self.model_cfg['in_dims'])  # size of this model input
        img_processed = np.expand_dims(img, axis=0).astype(self.model_cfg['input_type'])  # a,b,c -> 1,a,b,c
        return img_processed

    def run_network(self, img_preprocessed: np.array) -> None:
        self.interpreter.set_tensor(self.input_details[0]['index'], img_preprocessed)  # set input tensor
        self.interpreter.invoke()  # run
        return

    def extract_results(
            self,
            cv_img: np.array,
            fp: int = 2,
            ack: bool = False,
            tabs: int = 1,
            img_title: str = None
    ) -> list:
        """
        :param cv_img: cv image
        :param fp: float precision on the score precentage. e.g. fp=2: 0.1231231352353 -> 12.31%
        :param ack: if True: print meta data and detections
        :param tabs: if ack True: tabs for print
        :param img_title: if ack True: title for print
        :return: list of dicts:
        dict is a detection of an object above threshold.
            has items:
                label:str e.g. 'person'
                score_percentage: float e.g. 12.31
                bbox: dict with keys x0,y0,x1,y1
                #  pt1 = (x0, y0)  # obj frame top left corner
                #  pt2 = (x1, y1)  # obj frame bottom right corner
        """
        # get results
        boxes_np = self.interpreter.get_tensor(self.output_details[0]['index'])[0]  # bboxes
        labels_ids_np = self.interpreter.get_tensor(self.output_details[1]['index'])[0]  # labels as list of floats
        scores_np = self.interpreter.get_tensor(self.output_details[2]['index'])[0]  # confidence
        # count_np = interpreter.get_tensor(output_details[3]['index'])[0]  # number of detections

        if ack:
            title_suffix = '' if img_title is None else '{} '.format(img_title)
            title = '{} detections({}) on image {}{}:'.format(self.model_name, len(scores_np), title_suffix,
                                                              cv_img.shape)
            print('{}{}'.format(tabs * '\t', title))
            print('\n{}Meta_data(all detections):'.format(tabs * '\t'))
            print('{}\t{}'.format(tabs * '\t', mt.to_str(labels_ids_np, 'labels_ids_np')))
            print('{}\t{}'.format(tabs * '\t', mt.to_str(scores_np, 'scores')))
            print('{}\t{}'.format(tabs * '\t', mt.to_str(boxes_np, 'boxes')))

            print('\n{}Detections post first threshold and allowed_class:'.format(tabs * '\t'))
        if len(scores_np) <= 0:
            # no detections made
            if ack:
                print('{}\tNo detections found'.format(tabs * '\t'))
            return []
        boxes_nms_form_list = []  # x,y,w,h in proportion to the image, UNLIKE boxes_np which is x0,y0,x1,y1
        passed_first_filter = []  # threshold <= confidence and label in self.allowed_class
        img_h, img_w = cv_img.shape[0], cv_img.shape[1]
        for i, (bbox, label_id, confidence) in enumerate(zip(boxes_np, labels_ids_np, scores_np)):
            if self.model_cfg['threshold'] <= confidence <= 1.0 and label_id is not math.isnan(label_id):
                label = self.labels[int(label_id)]
                if label in self.allowed_classes:
                    # prepare box nms form and save index passed
                    center_x = int(bbox[0] * img_w)
                    center_y = int(bbox[1] * img_h)
                    w = int(bbox[2] * img_w)
                    h = int(bbox[3] * img_h)
                    x = center_x - w / 2
                    y = center_y - h / 2
                    boxes_nms_form_list.append([x, y, w, h])
                    passed_first_filter.append(i)
                    if ack:
                        msg = '{}\t{})Detected class {}: {}({:.2f}%)'
                        print(msg.format(tabs * '\t', i, int(label_id), label, confidence * 100))

        boxes_np = boxes_np[passed_first_filter]
        labels_ids_np = labels_ids_np[passed_first_filter]
        scores_np = scores_np[passed_first_filter]
        if len(scores_np) <= 0:
            if ack:
                print('{}\tNo detections passed first filter'.format(tabs * '\t'))
            return []
        if ack:
            print('{}Meta_data:'.format(tabs * '\t'))
            print('{}\t{}'.format(tabs * '\t', mt.to_str(labels_ids_np, 'labels_ids_np')))
            print('{}\t{}'.format(tabs * '\t', mt.to_str(scores_np, 'scores')))
            print('{}\t{}'.format(tabs * '\t', mt.to_str(boxes_np, 'boxes')))

        if self.model_cfg['nms'] is not None:  # activate nms
            # https://gist.github.com/leandrobmarinho/26bd5eb9267654dbb9e37f34788486b5
            passed_nms_filter = cv2.dnn.NMSBoxes(
                bboxes=boxes_nms_form_list,
                scores=scores_np.tolist(),
                score_threshold=self.model_cfg['nms']['score_threshold'],
                nms_threshold=self.model_cfg['nms']['nms_threshold']
            )
            if len(passed_nms_filter) > 0:
                passed_nms_filter = passed_nms_filter.flatten()
                boxes_np = boxes_np[passed_nms_filter]
                labels_ids_np = labels_ids_np[passed_nms_filter]
                scores_np = scores_np[passed_nms_filter]
            else:
                scores_np = []

            if ack:
                print('\n{}Meta_data(post-nms):'.format(tabs * '\t'))
                print('{}\t{}'.format(tabs * '\t', mt.to_str(passed_nms_filter, 'pick indices')))
                print('{}\t{}'.format(tabs * '\t', mt.to_str(labels_ids_np, 'labels_ids_np')))
                print('{}\t{}'.format(tabs * '\t', mt.to_str(scores_np, 'scores')))
                print('{}\t{}'.format(tabs * '\t', mt.to_str(boxes_np, 'boxes')))
                print('{}Detections:'.format(tabs * '\t'))
            if len(scores_np) <= 0:
                if ack:
                    print('{}\tNo detections passed nms filter'.format(tabs * '\t'))
                return []

        detections = []
        for i, (bbox, label_id, confidence) in enumerate(zip(boxes_np, labels_ids_np, scores_np)):
            if self.model_cfg['threshold'] <= confidence <= 1.0 and label_id is not math.isnan(label_id):
                label = self.labels[int(label_id)]
                if label in self.allowed_classes:
                    x0 = max(int(bbox[1] * img_w), 0)  # dont exceed 0
                    y0 = max(int(bbox[0] * img_h), 0)  # dont exceed 0
                    x1 = min(int(bbox[3] * img_w), img_w)  # dont exceed frame width
                    y1 = min(int(bbox[2] * img_h), img_h)  # dont exceed frame height
                    score_percentage = round(confidence * 100, fp)

                    detection_d = {
                        'label': label,
                        'score_percentage': score_percentage,
                        'bbox': {
                            #  pt1 = (x0, y0)  # obj frame top left corner
                            #  pt2 = (x1, y1)  # obj frame bottom right corner
                            'x0': x0,
                            'y0': y0,
                            'x1': x1,
                            'y1': y1,
                        },
                    }
                    detections.append(detection_d)
                    if ack:
                        d_msg = '{}\t{})Detected {}({}%) in top left=({}), bottom right=({})'
                        print(d_msg.format(tabs * '\t', i, label, score_percentage, (x0, y0), (x1, y1)))
        return detections

    def detect_cv_img(
            self,
            cv_img: np.array,
            fp: int = 2,
            ack: bool = False,
            tabs: int = 1,
            img_title: str = None
    ) -> list:
        """
        :param cv_img: open cv image
        :param fp: float precision on the score precentage. e.g. fp=2: 0.1231231352353 -> 12.31%
        :param ack: if True: print meta data and detections
        :param tabs: if ack True: tabs for print
        :param img_title: if ack True: title for print
        :return: list of dicts. see extract_results()
        """

        img_preprocessed = self.prepare_input(cv_img)
        self.run_network(img_preprocessed)
        detections = self.extract_results(
            cv_img=cv_img,
            fp=fp,
            ack=ack,
            tabs=tabs,
            img_title=img_title
        )
        return detections
